<?php

/**
 * @file
 * Functions for entities pages.
 */

function limesurvey_sync_api_synchronize_page($form, $form_state, $entity_type, $entity = FALSE) {

  $form = array();
  $all = FALSE;
  $count_need_sync = $count_need_import = $count_ok = $count_need_sync_ref = 0;
  $css = array(
    'css' => array(
      drupal_get_path('module', 'limesurvey_sync') . '/misc/limesurvey_sync.css',
    ),
  );
  $ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type);

  $form['#save_sync'] = FALSE;
  $form['#save_all'] = FALSE;
  if ($entity === FALSE) {
    // Synchronize all entities from $entity_type;
    $entity = entity_create($entity_type, array());
    $form['#save_all'] = TRUE;
    $all = 'all';
  }
  $form['#entity'] = $entity;

  // Test the sync.
  $synchronize_set = $entity->ls_synchronize($all);

  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];

  if (isset($synchronize_set['need_sync'])) {
    $count_need_sync = count($synchronize_set['need_sync']);
  }
  if (isset($synchronize_set['need_import']) && !in_array($entity_type, array('limesurvey_sync_survey', 'limesurvey_sync_response'))) {
    $count_need_import = count($synchronize_set['need_import']);
  }
  if (isset($synchronize_set['ok'])) {
    $count_ok = count($synchronize_set['ok']);
  }
  if (isset($synchronize_set['need_sync_ref'])) {
    $count_need_sync_ref = count($synchronize_set['need_sync_ref']);
  }
  $need_sync = ($ls_entity_type != 'survey') ? ($count_need_sync + $count_need_import + $count_need_sync_ref > 0) : ($count_need_sync > 0);

  if (!$form['#save_all']) {
    $arg = array(
      '@label' => entity_label($entity_type, $entity), //(isset($entity->title)) ? $entity->title : $entity->label,
    );
    $result = array(
      '#markup' => !$need_sync ? t('the @label is up to date.', $arg) : t('the @label is not synchronized : synchronize it.', $arg),
    );
    $description = t('The synchronization status between this Drupal site and your LimeSurvey site for the @label', $arg);
  }
  else {
    $arg = array(
      '@type' => $entity_info['label'],
      '@types' => $entity_info['plural label'],
    );
    $description = t('The synchronization status between this Drupal site and your LimeSurvey site for the @types', $arg);
    $n_entities = count(entity_load($entity_type));
    $all_update = ($n_entities < 1) ? t('No imported Drupal @type yet.', $arg) : ((($count_need_sync == 0) ? t('All already imported drupal @types are up to date. No updated need.', $arg) : format_plural($count_need_sync, '1 drupal @type need to be updated.', '@count drupal @types need to be updated.', $arg)));
    if ($ls_entity_type == 'survey') {
      // Manuel import for surveys.
      $result = array('#markup' => $all_update);
    }
    else {
      $result = array(
        '#theme' => 'item_list',
        '#items' => array(
          $all_update,
          (($count_need_import == 0) ? t('All LimeSurvey @types are present in this drupal site. No import needed.', $arg) : format_plural($count_need_import, '1 LimeSurvey @type need to be imported.', '@count LimeSurvey @types need to be imported.', $arg)),
        ),
      );
      if ($count_ok > 0) {
        $result['#items'][] = format_plural($count_ok, '1 @type is synchronized.', '@count @types are synchronized.', $arg);
      }
      if ($count_need_sync_ref > 0) {
        $result['#items'][] = format_plural($count_need_sync_ref, '1 @type needs synchronization on its answer or token references.', '@count @types need synchronization on its answer or token references.', $arg);
      }
    }
  }

  $form['sync_result'] = array(
    '#type' => 'fieldset',
    '#title' => t('Synchronization status'),
    '#description' => $description,
  );

  $form['sync_result']['result'] = array(
    '#prefix' => '<p><strong>',
    '#suffix' => '</strong></p>',
  );
  $form['sync_result']['result'] += $result;

  //Display need sync.
  $form['sync_result']['lss_sync'] = array(
    'data' => array(
      '#type' => 'markup',
      '#theme' => 'limesurvey_sync_field_label',
      '#field' => ($form['#save_all']) ? 'need_sync' : 'lss_sync',
      // Need sync has priority on 'to display'.
      '#to_display' => ($form['#save_all']) ? $need_sync : $entity->lss_sync,
      '#need_sync' => $need_sync,
      '#ls_entity_type' => $ls_entity_type,
      '#ls_sid' => $entity->ls_sid,
      '#attached' => $css,
      '#prefix' => '<p><strong>',
      '#suffix' => '</strong></p>',
    ),
  );


  // Display a table with the changes for updated entities.
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  if ($count_need_sync) {
    ksort($synchronize_set['need_sync']);
    $render = array();
    $ls_key_field = (!empty($entity_info['entity keys']['ls key'])) ? $entity_info['entity keys']['ls key'] : FALSE;
    $dp_key_field = (!empty($entity_info['entity keys']['id'])) ? $entity_info['entity keys']['id'] : FALSE;
    foreach ($synchronize_set['need_sync'] as $id => $datas) {
      // display the changes.
      $rows = $header = array();

      $first_row = TRUE;
      $row[] = array(
        'data' => (t('Drupal ID : @dp_id', array('@dp_id' => $datas['dp']->$dp_key_field)) . (($ls_key_field) ? ('<br />' . t('LimeSurvey ID : @ls_id', array('@ls_id' => $datas['dp']->$ls_key_field))) : '')),
        'rowspan' => count($datas['diff']) + (!empty($datas['ls_date']) ? 1 : 0),
      );

      if (!empty($datas['ls_date'])) {
        // Dates.
        $dp_date = format_date($datas['dp_date']);
        $ls_date = format_date($datas['ls_date']);
        $row[] = 'Last change on LimeSurvey';
        $row[] = $ls_date;
        $row[] = ($datas['ls_date'] == $datas['dp_date']) ? '=' : (($datas['ls_date'] > $datas['dp_date']) ? '>' : '<');
        $row[] = $dp_date;
        $rows[] = $row;
        $row = array();
      }
      $sync_entity = $datas['dp'];
      $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $ls_entity_type);
      foreach ($datas['diff'] as $field => $value) {

        if (!$first_row) {
          $row = array();
        }
        $first_row = FALSE;
        // $row[] = $id;
        $row[] = t('@field (@fieldname)', array('@field' => limesurvey_sync_api_display_field($ls_entity_type, $field, $value, 'field', $ls_sid), '@fieldname' => $field));
        // LS converted to drupal current value.
        $row[] = limesurvey_sync_api_display_field($ls_entity_type, $field, $value, 'value', $ls_sid);
        // Dp current value.
        $row[] = '-->';
        $row[] = limesurvey_sync_api_display_field($ls_entity_type, $field, $sync_entity->$field, 'value', $ls_sid);

        $rows[] = $row;
        $row = array();
      }

      // @TODO : need to theme thoses fields.
      $header[] = t('Id');
      $header[] = t('Field');
      $header[] = t('value on your LimeSurvey site');
      $header[] = '';
      $header[] = t('Value on this Drupal Site');

      $render[] = array(
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      );
    }

    $form['sync_result']['diff'] = array(
      'data' => $render,
    );
  }

  // Display a list with the LS ID to imort
  if ($count_need_import && $ls_entity_type != 'survey') {
    $items = $render_list = array();
    foreach ($synchronize_set['need_import'] as $ls_id => $datas) {
      $items[] = t('LimeSurvey @type #@id', array_merge($arg, array('@id' => $ls_id)));
    }
    $render_list[] = array(
      '#theme' => 'item_list',
      '#items' => $items,
    );
    $form['import_result']['text'] = array(
      '#markup' => t('@types to import', $arg),
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
    );
    $form['import_result']['diff'] = array(
      'data' => $render_list,
    );
  }

  if (isset($synchronize_set['need_sync_ref'])) {
    $need_sync = TRUE;
    // Look for synchronized responses but needing sync only on answer and token.
    $items = $render_list = array();
    foreach ($synchronize_set['need_sync_ref'] as $dp_rid => $datas) {
      $answer_and_token = (count($datas) >= 3);
      if ($answer_and_token) {
        $items[] = t('Response @dp_rid : answer and token need to be synchronized', array('@dp_rid' => $dp_rid));
      }
      else {
        $items[] = t('Response @dp_rid : @answer_or_token needs to be synchronized', array('@dp_rid' => $dp_rid, '@answer_or_token' => key($datas)));
      }
    }
    $render_list[] = array(
      '#theme' => 'item_list',
      '#items' => $items,
    );
    $form['sync_ref_result']['text'] = array(
      '#markup' => t('references to synchronize'),
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
    );
    $form['sync_ref_result']['diff_ref'] = array(
      'data' => $render_list,
    );
  }

  if ($need_sync) {
    $form['#save_sync'] = TRUE;
    // If need sync or import : display button to proceed.
    $form['actions'] = array(
      '#type' => 'actions',
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Synchronize'),
    );

  }

  return $form;
}

function limesurvey_sync_api_synchronize_page_submit($form, $form_state) {

  if (!$form['#save_sync']) {
    return;
  }
  $entity = $form['#entity'];

  $entity_type = limesurvey_sync_api_entityname($entity->ls_entity_type, $entity->ls_sid);
  $all = ($form['#save_all']) ? 'all' : FALSE;
  $synchronize_set = $entity->ls_synchronize($all, TRUE, TRUE);

  if (isset($synchronize_set['need_sync'])) {
    foreach ($synchronize_set['need_sync'] as $datas) {
      $entity_to_sync = $datas['dp'];
      $save_status = entity_save($entity_type, $entity_to_sync);
      //watchdog($entity_type, '@type: %title as been synchronized.', array(
//  '@type' => $entity_to_sync['dp']->ls_entity_type,
//  '%title' => $entity_to_sync['dp']->title
      //), WATCHDOG_NOTICE
      //current(entity_uri($entity_type, $entity_to_sync))
      //);
      drupal_set_message(t('The @type %title has been synchronized', array(
        '@type' => $entity_type,
        '%title' => $entity_to_sync->title,
      )));
    }
  }
  if (isset($synchronize_set['need_import']) && $form['#entity']->ls_entity_type != 'survey') {
    foreach ($synchronize_set['need_import'] as $ls_id => $entity_to_sync) {
      $status = entity_save($entity_type, $entity_to_sync);
      //watchdog($entity_type, '@type: %title as been imported.', array('@type' => $entity_to_sync->ls_entity_type, '%title' => $entity_to_sync->title), WATCHDOG_NOTICE);
      // current(entity_uri($entity_type, $entity_to_sync))
      drupal_set_message(t('The @type id #@id from the survey #@sid has been imported', array(
        '@id' => $ls_id,
        '@sid' => $entity->ls_sid,
        '@type' => $entity->ls_entity_type,
      )));

    }
  }

  if (isset($synchronize_set['need_sync_ref'])) {
    $items = $render_list = array();
    foreach ($synchronize_set['need_sync_ref'] as $dp_rid => $datas) {
      foreach ($datas as $answer_or_token => $ref_entities) {
        if ($answer_or_token == 'dp') {
          continue;
        }
        // $ref_entities is array(dp_key => entity);
        $ref_entity = current($ref_entities);
        reset($ref_entities);
        $answer_or_token_dp_id = key($ref_entities);
        // sync it.
        $entity_type_name = limesurvey_sync_api_entityname($answer_or_token, $ref_entity->ls_sid);
        $save_status = entity_save($entity_type_name, $ref_entity);
        $entity_info = entity_get_info($entity_type_name);
        $ls_field = $entity_info['entity keys']['ls key'];
        drupal_set_message(t('The @type id #@id from the response #@dp_rid has been synchronized', array(
          '@id' => $ref_entity->$ls_field,
          '@dp_rid' => $dp_rid,
          '@type' => $ref_entity->ls_entity_type,
        )));
      }
    }


  }
}

// Revision Stuff.
function limesurvey_sync_api_revision_overview($entity_type, $entity) {

  drupal_set_title(t('Revisions for %title', array(
    '%title' => $entity->title,
  )), PASS_THROUGH);

  $header = array(
    t('Revision'),
    array(
      'data' => t('Operations'),
      'colspan' => 2,
    ),
  );

  $ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type);
  $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $ls_entity_type);
  // Find survey.
  $survey_type = limesurvey_sync_api_typename('survey', $ls_sid);
  $survey = limesurvey_sync_survey_get_surveys($survey_type);
  $map = array(
    'view' => "view {$survey_type} {$ls_entity_type} revisions",
    'update' => "revert {$survey_type} {$ls_entity_type} revisions",
    'delete' => "delete {$survey_type} {$ls_entity_type} revisions",
  );

  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];
  $vid_field = $entity_info['entity keys']['revision'];
  $revisions = array();
  $result = db_query(
      'SELECT r.vid, r.lss_sync, r.lss_status, r.log, n.vid AS current_vid, r.timestamp
      FROM {' . $entity_info['revision table'] . '} r LEFT JOIN {' . $entity_info['base table'] . '} n ON n.' . $vid_field . ' = r.' . $vid_field . '
      WHERE r.' . $id_field . ' = :' . $id_field . '
      ORDER BY r.' . $entity_info['entity keys']['revision'] . ' DESC', array(
    ":{$id_field}" => $entity->$id_field,
  ));
  $revisions = $result->fetchAllAssoc($vid_field);

  $path = limesurvey_sync_api_root_uri($entity->ls_entity_type, $entity->ls_sid);
  $path_admin = limesurvey_sync_api_root_uri($entity->ls_entity_type, $entity->ls_sid, TRUE);
  $rows = array();
  $revert_permission = FALSE;
  // @TODO : In order to allow revision for survey and participant : need to
  // make a switch between "{$survey_type} {$ls_entity_type}" and
  // "{$ls_entity_type}" depending on the entity type.
  if ((user_access("revert {$survey_type} {$ls_entity_type} revisions") || user_access("administer limesurvey sync {$ls_entity_type}")) && entity_access('update', $entity_type, $entity)) {
    $revert_permission = TRUE;
  }
  $delete_permission = FALSE;
  if ((user_access("delete {$survey_type} {$ls_entity_type} revisions") || user_access("administer limesurvey sync {$ls_entity_type}")) && entity_access('delete', $entity_type, $entity)) {
    $delete_permission = TRUE;
  }
  foreach ($revisions as $revision) {
    $row = array();
    $operations = array();

    if ($revision->current_vid > 0) {
      $row[] = array(
        'data' => t('!date', array(
          '!date' => l(format_date($revision->timestamp, 'short'), "{$path}/{$entity->$id_field}"),
        )) . (($revision->log != '') ? '<p class="revision-log">' . filter_xss($revision->log) . '</p>' : ''),
        'class' => array(
          'revision-current',
        ),
      );
      $operations[] = array(
        'data' => drupal_placeholder(t('current revision')),
        'class' => array(
          'revision-current',
        ),
        'colspan' => 2,
      );
    }
    else {
      $row[] = t('!date', array(
        '!date' => l(format_date($revision->timestamp, 'short'), "{$path}/{$entity->$id_field}/revisions/{$revision->$vid_field}/view"),
      )) . (($revision->log != '') ? '<p class="revision-log">' . filter_xss($revision->log) . '</p>' : '');
      if ($revert_permission) {
        $operations[] = l(t('revert'), "{$path}/{$entity->$id_field}/revisions/{$revision->$vid_field}/revert");
      }
      if ($delete_permission) {
        $operations[] = l(t('delete'), "{$path}/{$entity->$id_field}/revisions/{$revision->$vid_field}/delete");
      }
    }
    $rows[] = array_merge($row, $operations);
  }

  $build['entity_revisions_table'] = array(
    '#theme' => 'table',
    '#rows' => $rows,
    '#header' => $header,
  );

  return $build;
}

function limesurvey_sync_api_revision_revert_confirm($form, $form_state, $entity_type, $entity_revision) {

  if (is_numeric($entity_revision)) {
    $entity_revision = entity_revision_load($entity_type, $entity_revision);
  }
  $form['#entity_revision'] = $entity_revision;
  // Look for changes on LS.
  $synchronization_set = $entity_revision->ls_synchronize('', 'dp2ls');
  $update_on_ls = (!empty($synchronization_set['need_sync']));
  $need_save_dp2ls = $update_on_ls;
  $description = ($update_on_ls) ? '<strong>' . t('The corresponding entry into the LimeSurvey site will be updated accordingly.') . '</strong>' : '';
  // Save it into $form (and not $form_state) for the submit form.
  $form['need_save_dp2ls'] = array(
    '#type' => 'value',
    '#value' => $need_save_dp2ls,
  );

  $url = entity_uri($entity_type, $entity_revision);
  $url['path'] = $url['path'] . '/revisions';
  return confirm_form($form, t('Are you sure you want to revert to the @type revision from %revision-date?', array(
    '@type' => $entity_revision->ls_entity_type,
    '%revision-date' => format_date($entity_revision->timestamp),
  )), $url, $description, t('Revert'), t('Cancel'));
}

function limesurvey_sync_api_revision_revert_confirm_submit($form, &$form_state) {

  $entity_revision = $form['#entity_revision'];
  $need_save_dp2ls = (isset($form['need_save_dp2ls']['#value'])) ? $form['need_save_dp2ls']['#value'] : FALSE;

  $entity_type = limesurvey_sync_api_entityname($entity_revision->ls_entity_type, $entity_revision->ls_sid);
  $entity_info = entity_get_info($entity_type);
  $base_field = $entity_info['entity keys']['id'];
  $vid_field = $entity_info['entity keys']['revision'];
  $entity_revision->revision = 1;
  $entity_revision->log = t('Copy of the revision from %date.', array(
    '%date' => format_date($entity_revision->timestamp),
  ));
  entity_revision_set_default($entity_type, $entity_revision);
  // Flag it to not be synchronized !
  $entity_revision->need_save_ls2dp = FALSE;
  // Flag it to update or not the LS entry into LS.
  $entity_revision->need_save_dp2ls = $need_save_dp2ls;
  // Save the revision.
  entity_save($entity_type, $entity_revision);

  watchdog($entity_type, '@type: reverted %title revision %revision.', array(
    '@type' => $entity_revision->ls_entity_type,
    '%title' => $entity_revision->title,
    '%revision' => $entity_revision->$vid_field,
  ));
  drupal_set_message(t('@type %title has been reverted back to the revision from %revision-date.', array(
    '@type' => $entity_info['label'],
    '%title' => $entity_revision->title,
    '%revision-date' => format_date($entity_revision->timestamp),
  )));
  $path = limesurvey_sync_api_root_uri($entity_revision->ls_entity_type, $entity_revision->ls_sid, FALSE);
  $form_state['redirect'] = $path . '/' . $entity_revision->$base_field . '/revisions';
}

function limesurvey_sync_api_revision_delete_confirm($form, $form_state, $entity_type, $entity_revision) {

  if (is_numeric($entity_revision)) {
    $entity_revision = entity_revision_load($entity_type, $entity_revision);
  }
  $ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type);
  $form['#entity_revision'] = $entity_revision;
  $url = entity_uri($entity_type, $entity_revision);
  $url['path'] = $url['path'] . '/revisions';
  return confirm_form($form, t('Are you sure you want to delete the @type revision from %revision-date?', array(
    '@type' => $entity_revision->ls_entity_type,
    '%revision-date' => format_date($entity_revision->timestamp),
  )), $url, t('This action cannot be undone.'), t('Delete'), t('Cancel'));
}

function limesurvey_sync_api_revision_delete_confirm_submit($form, &$form_state) {

  $entity_revision = $form['#entity_revision'];
  $entity_type = limesurvey_sync_api_entityname($entity_revision->ls_entity_type, $entity_revision->ls_sid);
  $entity_info = entity_get_info($entity_type);
  $base_field = $entity_info['entity keys']['id'];
  $vid_field = $entity_info['entity keys']['revision'];
  entity_revision_delete($entity_type, $entity_revision->$vid_field);

  watchdog($entity_type, '@type: deleted %title revision %revision.', array(
    '@type' => $entity_revision->ls_entity_type,
    '%title' => $entity_revision->title,
    '%revision' => $entity_revision->$vid_field,
  ));
  drupal_set_message(t('Revision from %revision-date of @type %title has been deleted.', array(
    '%revision-date' => format_date($entity_revision->timestamp),
    '@type' => $entity_revision->ls_entity_type,
    '%title' => $entity_revision->title,
  )));
  $path = limesurvey_sync_api_root_uri($entity_revision->ls_entity_type, $entity_revision->ls_sid, FALSE);
  $form_state['redirect'] = $path . '/' . $entity_revision->$base_field;
  if (db_query('SELECT COUNT(' . $vid_field . ') FROM {' . $entity_info['revision table'] . '} WHERE ' . $base_field . ' = :' . $base_field, array(
    ':' . $base_field => $entity_revision->$base_field,
  ))->fetchField() > 1) {
    $form_state['redirect'] .= '/revisions';
  }
}
