<?php
/**
 * @file
 * Return page from the LimeSurvey site, after answer completion.
 */

/**
 * Handle synchronization on coming back from the LimeSurvey site to Drupal.
 *
 * Try to retreive the edited answer via several method. At the end, redirect
 * to a specific page (it can be the synchronized answer node view page) or the
 * front page if it fails. Note that permission access to this page is set to
 * 'access content' : this page should not print informations without testing
 * them on rights.
 *
 * @param $type
 *   A string, the answer content type type.
 * @param $back
 *   A boolean, TRUE if the previous page was the iframe page or FALSE if it
 *   was the LimeSurvey form page. Default is FALSE.
 */
function limesurvey_sync_invitation_back_from_ls($survey, $dp_rid = NULL) {

  // check if the frame is activated.
  //$http_referer = $_SERVER['HTTP_REFERER'];
  //watchdog('limesurevy_sync', 'debug http_referer from iframe : @referer', array('@referer' => $http_referer));
  // Check if auto_sync is set.
  //$allowed_auto_import = ($survey_properties['date_answer'] && !$survey_properties['anonymized']);
  //$editable_surveys = ($survey_properties['active'] && $survey_properties['is_available']);
  //if ($options['ls_auto_import'] && $allowed_auto_import && $editable_surveys) {

  $type = $survey->type;
  $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_typename($type, 'survey');

  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $survey_properties = limesurvey_sync_survey_properties($ls_sid);

  // Set initial parameters :
  // Set an arbitrary duration in seconds for filling the survey from the start date from DP to its final submission.
  // The system will explore all dp opened response from that point to now.
  $duration_filling = 1500; // Arbitrary set it to 25min.
  $duration_filling_token = 3600; // Same but if the token can be retrieved into the refererrer url.
  // Set delay between LS final submission and arriving to this dp page.
  $ls_delay_completed = ($survey_properties['autoredirect']) ? 20 : 180;

  // Load survey datas.
  $variable_name = "limesurvey_sync_survey_{$ls_sid}_options";
  $ls_options = variable_get($variable_name, array());
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_invitation.options');
  $options = limesurvey_sync_invitation_options_default($ls_sid, $ls_options);
  $display_mode = $options['ls_redirect'];
  // Set the correct display mode.
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_invitation.form');
  $display_mode = limesurvey_sync_invitation_fix_display_mode($display_mode);

  // Load JS for closing frame if any.
  limesurvey_sync_invitation_close_frames($display_mode);

  // Set default redirection.
  $end_url = (!empty($options['ls_end_url'])) ? $options['ls_end_url'] : '<front>';

  $response = $token_method = FALSE;
  $response_entityname = limesurvey_sync_api_entityname('response', $ls_sid);
  $survey_typename = limesurvey_sync_api_typename('survey', $ls_sid);
  $answer_entityname = limesurvey_sync_api_entityname('answer', $ls_sid);
  $answer_table = limesurvey_sync_api_tablename('answer', $ls_sid);

  // if come from the iframe.
  $come_from_iframe = FALSE;
  if (!empty($dp_rid)) {
    $come_from_iframe = TRUE;
    $response = current(entity_load($response_entityname, array($dp_rid)));
  }

  if (empty($response) && $survey_properties['tokens_table'] && ($survey_properties['tokenanswerspersistence'] || $survey_properties['alloweditaftercompletion'])) {
    // Select a method to retrieve the fresh edited answer.
    if ($survey_properties['http_referer']) { // && !in_array($options['ls_redirect'], $iframes) && !$back) {
      // Search datas into the http_referer value.
      // The http_referer method should work only if :
      // - "Show welcome screen?" is set to No,
      // - and "Automatically load URL when survey complete?" is set to Yes,
      // - and it is a one page survey.
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
      $extract = limesurvey_sync_ls_api_answer_extract_referer();
      if ($extract && is_array($extract) && isset($extract['ls_sid']) && $extract['ls_sid'] == $ls_sid) {
        if (!empty($extract['ls_token'])) {
          // Token is provided by the http referer url.
          $ls_token = $extract['ls_token'];
          // Select it if the submission was between now and 1h ago (arbitrary fixed).
          $duration = REQUEST_TIME - $duration_filling_token;
          // look for an answer with this token
          $find_dp_aid = limesurvey_sync_api_response_retrieve_dp_aid_from_token($ls_sid, $ls_token);
          if ($find_dp_aid['dp_aid'] && $find_dp_aid['need_sync'] && $find_dp_aid['ls_submitdate'] > $duration) {
            // An answer with this token does exists on LS, as been edited soon and need sync.
            // Retrieve the corresponding token entity.
            $token_table = limesurvey_sync_api_tablename('token', $ls_sid);
            $dp_tid = db_query('SELECT dp_tid FROM {' . $token_table . '} WHERE ls_token = :ls_token ORDER BY changed DESC', array(':ls_token' => $ls_token))->FetchField();
            // @TODO : make sure there is only one entity token.
            if ($dp_tid) {
              // Retrieve the response entity associated to this token entity.

              $token_field_name = limesurvey_sync_api_et_fieldname('token', $ls_sid);
              $query = new EntityFieldQuery();
              $query->entityCondition('entity_type', $response_entityname)
              ->entityCondition('bundle', $survey_typename)
              ->fieldCondition($token_field_name, 'target_id', $dp_tid, '=' )
              ->range(0, 10)
              ->addMetaData('account', user_load(1)); // Run the query as user 1.

              $result = $query->execute();
              if (is_array($result) && count($result[$response_entityname]) == 1) {
                $dp_rid = key($result[$response_entityname]);
              }
            }

            $responses = entity_load($response_entityname, array($dp_rid));
            if ($responses) {
              $response = $responses[$dp_rid];
              $token_method = TRUE;
              // Save the response below.
            }
          }
        }

      }
    }
  }

  if (empty($response)) {
    // Find with another way.
    // Look for recent completed answer into the LS database.
    // @TODO : disabled it for anonymized LS survey.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
    $recent_ls_answers = limesurvey_sync_ls_api_answer_retrieve_recent_completed($ls_sid, $ls_delay_completed);
    if (!empty($recent_ls_answers)) {
      // There is/are recently completed answer(s) on LS.
      // Look for answers asked from DP for that user.
      // Retrieve the token
      global $user;
      $arg = array(
        ':type' => $type,
        ':revision_uid' => $user->uid,
        ':dp_editing' => REQUEST_TIME - $duration_filling,
        ':dp_back' => 0,
      );
      $response_table = limesurvey_sync_api_tablename('response', $ls_sid);
      $response_table_revision = limesurvey_sync_api_tablename('response', $ls_sid, TRUE);
      $result = db_query("SELECT revision.dp_rid AS dp_rid, revision.revision_uid AS revision_uid, revision.dp_editing AS dp_editing, revision.dp_back AS dp_back FROM $response_table base INNER JOIN $response_table_revision revision ON revision.vid = base.vid WHERE
            revision.type = :type
            AND revision_uid = :revision_uid
            AND revision.dp_editing > :dp_editing
            AND revision.dp_back = :dp_back
            ORDER BY revision.dp_editing DESC", $arg);
      $recent_responses = array();
      while ($recent_response = $result->fetchAssoc()) {
        $recent_responses[] = $recent_response['dp_rid'];
      }
      if (!empty($recent_responses)) {
        // There are recent response "opened" for this userand survey.
        // Load all those recent responses.
        $responses = entity_load($response_entityname, $recent_responses);
      }

      // Load recent submited answers from LS.
      $ls_answer_to_dp_id = array();
      $answer_field_name = limesurvey_sync_api_et_fieldname('answer', $ls_sid);
      $answer_get_info = entity_get_info($answer_entityname);
      $ls_id_col = $answer_get_info['entity keys']['ls key'];
      //$answer_table = limesurvey_sync_api_tablename('answer', $ls_sid);
      $ls_dp_aid_by_ls_id = array();
      foreach ($recent_ls_answers as $ls_id => $ls_answer) {
        // Look for an associated answer on drupal.
        $ls_lang = $ls_answer['ls_startlanguage'];
        $dp_aid = db_query("SELECT {$answer_get_info['entity keys']['id']} FROM $answer_table base WHERE $ls_id_col = :ls_id  ORDER BY {$answer_get_info['entity keys']['id']} DESC", array(':ls_id' => $ls_id))->fetchField();
        if ($dp_aid) {
          // This answer has been imported into DP.
          $ls_dp_aid_by_ls_id[$ls_id] = $dp_aid;
          // look for an associated response to this dp answer.
          $query = new EntityFieldQuery();
          $query->entityCondition('entity_type', $response_entityname)
              ->entityCondition('bundle', $survey_typename)
              ->fieldCondition($answer_field_name, 'target_id', $dp_aid, '=' )
              ->range(0, 10)
              ->addMetaData('account', user_load(1)); // Run the query as user 1.

          $result = $query->execute();
          if (is_array($result) && isset($result[$response_entityname]) && count($result[$response_entityname]) == 1) {
            $dp_rid = key($result[$response_entityname]);
            $ls_answer_to_dp_id['associated'][$ls_lang][$ls_id] = $dp_rid;
          }
          else {
            $ls_answer_to_dp_id['not_associated'][$ls_lang][$ls_id] = $dp_aid;
          }
        }
        else {
          $ls_answer_to_dp_id['not_associated'][$ls_lang][$ls_id] = NULL;
        }
      }

      // $responses_by_ls_id : refences indexed by ls_id
      $responses_by_ls_id = array();
      $dp_dp_aid_by_dp_id = array();
      foreach ($responses as $dp_rid => $dp_response) {
        $answer_field = $dp_response->$answer_field_name;
        $ls_lang = $dp_response->ls_language;
        if (isset($answer_field[LANGUAGE_NONE][0]['target_id']) && !empty($answer_field[LANGUAGE_NONE][0]['target_id'])) {
          $dp_aid = $answer_field[LANGUAGE_NONE][0]['target_id'];
          $dp_dp_aid_by_dp_id[$dp_rid] = $dp_aid;
          $answer = node_load($answer_entityname, array($dp_aid));
          if ($answer) {
            $responses_by_ls_id['associated'][$ls_lang][$answer->$ls_id_col] = $dp_response->dp_rid;
          }
          else {
            // The response is bounded to an answer but this dp answer has been deleted on DP.
            $responses_by_ls_id['not_associated'][$ls_lang][$dp_response->dp_rid] = $dp_aid;
          }
        }
        else {
          $responses_by_ls_id['not_associated'][$ls_lang][$dp_response->dp_rid] = NULL;
        }
      }

      // Now compare both LS and Drupal.
      $matching = array();
      if (isset($ls_answer_to_dp_id['associated'], $responses_by_ls_id['associated'])) {
        foreach ($ls_answer_to_dp_id['associated'] as $ls_lang => $ls_ids) {
          foreach ($ls_ids as $ls_id => $dp_rid) {
            if (isset($responses_by_ls_id['associated'][$ls_lang][$ls_id]) && $responses_by_ls_id['associated'][$ls_lang][$ls_id] == $dp_rid) {
              // This recent LS answer match a recent DP response.
              $matching[$dp_id] = $ls_id;
            }
          }
        }
      }

      if (!empty($matching) && count($matching) == 1) {
        $response = $responses[key($matching)];
        $ls_id = current($matching);
      }
      elseif (!empty($matching)) {
        // @TODO : Look for a better specific way.
        // Pick one.
        $response = $responses[key($matching)];
        $ls_id = current($matching);
      }
      else {
        // Look for the answer to import.
        if (isset($ls_answer_to_dp_id['not_associated'])) {
          $delta_editing_stardate = array();
          foreach ($ls_answer_to_dp_id['not_associated'] as $ls_lang => $ls_ids) {
            foreach ($ls_ids as $ls_id => $dp_aid) {
              if (isset($responses_by_ls_id['not_associated'][$ls_lang])) {
                $ls_answer = $recent_ls_answers[$ls_id];

                foreach ($responses_by_ls_id['not_associated'][$ls_lang] as $dp_rid => $dp_aid) {

                  $dp_response = $responses[$dp_rid];
                  if ($ls_answer['ls_submitdate'] > $dp_response->dp_editing && $ls_answer['ls_startdate'] >= $dp_response->dp_editing) {
                    $matching[$dp_rid] = $ls_id;
                    $delta_editing_stardate[$ls_answer['ls_startdate'] - $dp_response->dp_editing][$dp_rid] = $ls_id;
                  }
                }

              }
            }
          }
          if (!empty($matching) && count($matching) == 1) {
            $response = $responses[key($matching)];
            $ls_id = current($matching);
          }
          elseif (!empty($matching)) {
            // Several possibilities. Select one where ls_startdate and dp_editind are nearest.
            ksort($delta_editing_stardate);
            // pick the first one.
            $matching_from_duration = array_shift($delta_editing_stardate);
            $dp_rid_final = key($matching_from_duration);
            $response = $responses[$dp_rid_final];
            $ls_id = $matching_from_duration[$dp_rid_final];
          }
          else {
            // Not founded.
          }
        }
      }
    }
  }

  if ($response && !$come_from_iframe) {
    // The response and ls_id is founded.
    // Check if the ls answer need to be imported.
    if (!$token_method) {
      if (!empty($ls_dp_aid_by_ls_id[$ls_id])) {
        $dp_aid = $ls_dp_aid_by_ls_id[$ls_id];
      }
      else {
        // Create the answer entity.
        $answer_entity = entity_create($answer_entityname, array('ls_id' => $ls_id));
        entity_save($answer_entityname, $answer_entity);
        $dp_aid = $answer_entity->dp_aid;
      }
      // Set the response entity with the dp_aid.
      if (!isset($response->{$answer_field_name})) {
        $response->{$answer_field_name} = array();
      }
      $response->{$answer_field_name}[LANGUAGE_NONE][0]['target_id'] = $dp_aid;
      $response->{$answer_field_name}[LANGUAGE_NONE][0]['target_type'] = $answer_entityname;
    }
    // Flag the response entity.
    $response->dp_backfromls = TRUE;
    // Save the finded response.
    entity_save($response_entityname, $response);

  }

  if ($response) {
    // redirect t the response page if desired.

    if ($options['ls_end'] == 'own' && entity_access('view', $response_entityname, $response)) {
      $entity_url_array = entity_uri($response_entityname, $response);
      $end_url = $entity_url_array['path'];
    }
  }
  // Redirect to the answer node view page or a specific page or the front
  // page.
  if (!$come_from_iframe) {
    // JS redirection. This allow the page to be displayed, so the child JS close the parent iframe. Then the redirection occures.
    $redirection_timing = 100; //ms.
    $redirection_url = url($end_url, array('absolute' => TRUE));
    $js_redirect = "setTimeout(\"window.location='$redirection_url'\", $redirection_timing);";
    $js_script = limesurvey_sync_invitation_js_autoload_prepare($js_redirect);
    drupal_add_js($js_script, array('type' => 'inline', 'scope' => 'header', 'group' => JS_DEFAULT, 'weight' => 6));
    return t('Redirection in progress...');
  }
  else {
    drupal_goto($end_url);
  }
  return t('Redirection in progress...');
}
