<?php
/**
 * @file
 * fonctions managing options on the survey edit page.
 */

/**
 * Provide options default values for a survey ID, or the provided options datas
 * completed by the missing default values.
 *
 * @param $ls_sid
 *   A numeric (optional). The survey ID. Default is NULL
 * @param $answer_options
 *   An array (optional). The options datas.
 *
 * @return
 *   An array of the each option value.
 */
function limesurvey_sync_invitation_options_default($ls_sid = NULL, $answer_options = array()) {
  $answer_options = (array) $answer_options;
  $answer_options += array(
    'ls_redirect' => 'redirect',
    'ls_overwrite_endurl' => 0,
    'iframe_dim' => 'auto',
    'iframe_dim_width' => '100%',
    'iframe_dim_height' => 700,
    'ls_end' => 'page',
    'ls_end_url' => '',
    'ls_hide_lang' => 0,
    'ls_import_dp2ls_datas' => 0,
    'email' => 'mail',
    'firstname' => 'none',
    'lastname' => 'name',
    'usesleft' => 1,
    'use_secure_token' => 1,
    'secure_validuntil_period' => 7200,
    'secure_validuntil_period_custom' => '',
    'ls_prefix' => '',
    'ls_suffix_method' => 'numeric',
    'ls_suffix_min' => 1000,
    'ls_suffix_max' => 9999,
    'ls_disabled_title' => 0,
  );
  // Add 'attribute_1, attribute_2, etc...
  if ($ls_sid) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($ls_sid);
    if ($this_survey) {
      $attributedescriptions = $this_survey['attributedescriptions'];
      foreach ($attributedescriptions as $attribute_field => $datas) {
        if (!array_key_exists($attribute_field, $answer_options)) {
          $answer_options[$attribute_field] = 'none';
        }
      }
    }
  }

  return $answer_options;
}

/**
 * Form builder for the survey options.
 *
 * Set survey options on specific fieldsets.
 *
 * @see limesurvey_sync_invitation_options_validate()
 * @see limesurvey_sync_invitation_options_form_submit()
 * @see limesurvey_sync_invitation_options_redirect()
 * @ingroup forms
 */
function limesurvey_sync_invitation_options_form($form, $form_state, $ls_id = NULL) {

  form_load_include($form_state, 'inc', 'limesurvey_sync', 'survey/limesurvey_sync_survey.options');
  // $form_state['entity_type'] is set by the entity_ui_get_form() function.
  $survey_entityname = limesurvey_sync_api_entityname('survey');
  $default_options_page = (empty($form_state['entity_type']) || $form_state['entity_type'] != $survey_entityname);
  // Load appropriate variable.
  if (!$default_options_page) {
    // Load survey datas.
    $ls_sid = (!empty($form_state['values']['ls_sid'])) ? $form_state['values']['ls_sid'] : ((!empty($ls_id)) ? $ls_id : FALSE);
    $survey = limesurvey_sync_survey_get_survey_by_sid($ls_id);
    $new_survey = (!$survey);

    $variable_name = "{$survey_entityname}_{$ls_sid}_options";
    $ls_options = variable_get($variable_name, array());
    // Add missing options if any.
    $answer_options = limesurvey_sync_invitation_options_default($ls_sid, $ls_options);
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = ($ls_sid) ? limesurvey_sync_survey_properties($ls_sid) : FALSE;
    $survey_label = (!empty($form_state['values']['label'])) ? $form_state['values']['label'] : ($survey && (!empty($survey->label)) ? $survey->label : t('the survey name'));
  }
  else {
    $ls_sid = $survey_properties = FALSE;
    $answer_options = variable_get('limesurvey_sync_survey_default_options', limesurvey_sync_invitation_options_default());
    $survey_label = t('the survey name');
  }

  $form['ls_invitation_options'] = array(
    '#type' => 'fieldset',
    '#title' => ($default_options_page) ? t('Options for defaut survey responses') : t('Options when a user answer to the survey %sid from your Drupal site.', array('%sid' => $ls_sid)),
    '#description' => ($default_options_page) ? t('Choose display options for defaut survey responses') : t('Choose display options for this survey responses'),
    '#weight' => 0,
    // Put the validate function into the hook_form_alter instead to make the
    // #node_type variable available for the validate function.
    // Avoid to call twice this function because of the $form_id . '_validate' hook.
    '#element_validate' => array('limesurvey_sync_invitation_options_form__validate'),
  );

  $redirect_method = array(
    //'link' => t('print a simple hypertext link'),
    //'form' => t('print a form button'),
    'redirect' => t('automatic and immediate redirection'),
    'iframe' => t('open into an iframe'),
    'colorbox' => t("open into a lightbox from the '<a href=\"http://drupal.org/project/colorbox\">colorbox</a>' module (dimensions can be set using percents or pixels)"),
    'lightbox2' => t("open into a lightbox from the '<a href=\"http://drupal.org/project/lightbox2\">lightbox2</a>' module (dimensions can be set using pixels only)"),
    // The modalframe is Drupal6 only (D7 port: http://drupal.org/node/491224).
    //'modalframe' => t("open into a Modalframe frame from the '<a href=\"http://drupal.org/project/modalframe\">Modal Frame API</a>' module"),
  );

  $form['ls_invitation_options']['answ_redirect'] = array(
    '#type' => 'fieldset',
    '#title' => t('Options for the response completing form'),
    '#description' => t('Users submits their answers <em>on</em> the LimeSurvey website. Choose options about the completing form.'),
    '#collapsible' => 1,
    '#collapsed' => 0,
  );
  $message = t('In order to automatically close the frame, you should :');
  $form['ls_invitation_options']['answ_redirect']['ls_redirect'] = array(
    '#type' => 'radios',
    '#title' => t('Choose the way to redirect the user to the LimeSurvey site, in order to fill the survey'),
    '#default_value' => $answer_options['ls_redirect'],
    '#options' => $redirect_method,
    '#description' => $message . limesurvey_sync_invitation_options_help_iframe($ls_sid, TRUE),
  );
  $form['ls_invitation_options']['answ_redirect']['ls_overwrite_endurl'] = array(
    '#type' => 'checkbox',
    '#title' => t('Automatically overwrite the LimeSurvey end url into the LimeSurvey database, in order to reflect the above settings.'),
    '#default_value' => $answer_options['ls_overwrite_endurl'],
    '#return_value' => 1,
    '#description' => t('Or edit it manually from the LimeSurvey admin interface.'),
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim'] = array(
    '#type' => 'item',
    '#title' => t('Choose the iframe/lightbox dimensions'),
    '#description' => t("Set '0' to automatically fit the frame on one dimension."),
    '#states' => array('visible' => array(':input[name="ls_redirect"]' => array(
          array('value' => 'iframe'),
          array('value' => 'colorbox'),
          array('value' => 'lightbox2'),
        ))),
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['auto'] = array(
    '#type' => 'radio',
    '#title' => t('automatically fit to the user window'),
    '#default_value' => $answer_options['iframe_dim'],
    '#return_value' => 'auto',
    '#parents' => array('iframe_dim'),
    // The first radio button needs to make the style match a normal radios
    // group.
    '#prefix' => '<div class="form-radios">',
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['invisible'] = array(
    '#type' => 'radio',
    '#title' => t('Invisible iframe ! No vertical scrolling bar into on the iframe') . ' (' . t('beta') . ')',
    '#default_value' => $answer_options['iframe_dim'],
    '#return_value' => 'invisible',
    '#parents' => array('iframe_dim'),
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['iframe_dim_group'] = array(
    '#prefix' => '<div class="container-inline form-item">',
    '#suffix' => '</div>',
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['iframe_dim_group']['cutom'] = array(
    '#type' => 'radio',
    '#title' => t('Use thoses dimensions :'),
    '#default_value' => $answer_options['iframe_dim'],
    '#return_value' => 'custom',
    '#parents' => array('iframe_dim'),
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['iframe_dim_group']['iframe_dim_width'] = array(
    '#type' => 'textfield',
    '#default_value' => $answer_options['iframe_dim_width'],
    '#required' => 0,
    '#size' => 8,
    '#title' => t('width'),
    '#suffix' => t('px or %') . '; ',
  );
  $form['ls_invitation_options']['answ_redirect']['iframe_dim']['iframe_dim_group']['iframe_dim_height'] = array(
    '#type' => 'textfield',
    '#default_value' => $answer_options['iframe_dim_height'],
    '#required' => 0,
    '#size' => 8,
    '#title' => t('height'),
    '#suffix' => t('px or %') . '</div>',
  );

  $end_url = array(
    'own' => t('its answer view page'),
    'page' => t('this specific page :'),
  );
  // In order to use the core sentence, remove double em tag.
  $right = filter_xss(t('%type_name: View own response and View own answer', array('%type_name' => $survey_label)), array());
  $url_perm = url('admin/people/permissions');
  $form['ls_invitation_options']['answ_redirect']['ls_end'] = array(
    '#type' => 'item',
    '#title' => t('Once the form is submitted, redirect the user to'),
    '#description' => t('In order to allow a user to view its submission datas, you need to <b>set the appropiate rights</b> (%right) on the <a href="@url">permission setting page</a>', array('%right' => $right, '@url' => $url_perm)),
  );
  $form['ls_invitation_options']['answ_redirect']['ls_end']['own'] = array(
    '#type' => 'radio',
    '#title' => t('its response view page'),
    '#default_value' => $answer_options['ls_end'], // If there is a default value, this should also be specified for each radio button.
    '#return_value' => 'own',
    '#parents' => array('ls_end'), // You must specify this for each radio button for them to act as a group.
    '#prefix' => '<div class="form-radios">', // The first radio button needs to make the style match a normal radios group.
  );
  $form['ls_invitation_options']['answ_redirect']['ls_end']['page_group'] = array(
    '#prefix' => '<div class="container-inline form-item">',
    '#suffix' => '</div>',
  );
  $form['ls_invitation_options']['answ_redirect']['ls_end']['page_group']['page'] = array(
    '#type' => 'radio',
    '#title' => t('this specific page :'),
    '#default_value' => $answer_options['ls_end'],
    '#return_value' => 'page',
    '#parents' => array('ls_end'),
  );
  $end_url_length = drupal_strlen($answer_options['ls_end_url']);
  $form['ls_invitation_options']['answ_redirect']['ls_end']['page_group']['ls_end_url'] = array(
    '#type' => 'textfield',
    '#default_value' => $answer_options['ls_end_url'],
    '#description' => 'For example : node/458, leave it blank for the home page.',
    '#required' => 0,
    '#size' => ($end_url_length == 0) ? 20 : ($end_url_length + 10),
    '#suffix' => '</div>',
  );
  $form['ls_invitation_options']['answ_redirect']['ls_hide_lang'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide the languages dropdown on the Drupal page preceding the LimeSurvey survey form page.'),
    '#default_value' => $answer_options['ls_hide_lang'],
    '#return_value' => 1,
    '#description' => t("If the 'Locale' core module is enabled. The default language is set below."),
  );

  if ($default_options_page) {
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save options'),
      '#weight' => 40,
    );
    //$form['#validate'][] = 'limesurvey_sync_invitation_options_form_validate';
    //$form['default_options']['ls_invitation_options'] = $form['ls_invitation_options'];
    //unset($form['ls_invitation_options']);
  }
  // The 'limesurvey_sync_invitation_options_form_submit()' function will be moved by the hook
  // form alter in order to be executed before the node_type_form_submit()
  // function.
  //$form['#submit'][] = 'limesurvey_sync_invitation_options_form_submit';

  $form['ls_invitation_options']['answ_title'] = array(
    '#type' => 'fieldset',
    '#title' => t('The respond title'),
    '#description' => t('Choose how the title associated to each response will be created.'),
    '#collapsible' => TRUE,
    //'#group' => 'additional_settings',
  );
  $ls_prefix = $answer_options['ls_prefix'];
  $ls_prefix_size = (!empty($ls_prefix)) ? drupal_strlen($ls_prefix) + 10 : 25;
  if (!$default_options_page && $new_survey && (empty($form_state['values']['ls_prefix'])) && empty($answer_options['ls_prefix'])) {
    // On survey creation, 'Response #' will be overwrited by JS corresponding
    // to the LS survey label (default language).
    $ls_prefix = t('Response #');
    $ls_prefix_size = 60;
  }
  $form['ls_invitation_options']['answ_title']['ls_prefix'] = array(
    '#type' => 'textfield',
    '#title' => t('Prefix for the response title'),
    '#default_value' => $ls_prefix,
    '#description' => 'Delete the value for an empty title',
    '#required' => 0,
    '#size' => $ls_prefix_size,
  );
  $form['ls_invitation_options']['answ_title']['suffix'] = array(
    '#type' => 'item',
    '#title' => t('Suffix, to identify a single response'),
  );
  $form['ls_invitation_options']['answ_title']['suffix']['numeric'] = array(
    '#type' => 'radio',
    '#title' => t('An increment number'),
    // The default value should also be specified for each radio button.
    '#default_value' => $answer_options['ls_suffix_method'],
    '#return_value' => 'numeric',
    // You must specify this for each radio button for them to act as a group.
    '#parents' => array('ls_suffix_method'),
    // The first radio button needs to make the style match a normal radios group.
    '#prefix' => '<div class="form-radios">',
  );
  $form['ls_invitation_options']['answ_title']['suffix']['random_group'] = array(
    '#prefix' => '<div class="container-inline form-item">',
    '#suffix' => '</div>',
  );
  $form['ls_invitation_options']['answ_title']['suffix']['random_group']['random'] = array(
    '#type' => 'radio',
    '#title' => t('A random number'),
    '#default_value' => $answer_options['ls_suffix_method'],
    '#return_value' => 'random',
    '#parents' => array('ls_suffix_method'),
  );
  $form['ls_invitation_options']['answ_title']['suffix']['random_group']['ls_suffix_min'] = array(
    '#type' => 'textfield',
    '#title' => t('Minimal number'),
    '#default_value' => $answer_options['ls_suffix_min'],
    '#size' => '5',
    '#maxlength' => '10',
    // shown : states doesn't work. set it as markup.
    //'#prefix' => ' ' . t('between :'),
    //'#suffix' => ' ' . t('and') . ' ',
    '#states' => array(
      // Hide the settings.
      'visible' => array(
        ':input[name="ls_suffix_method"]' => array('value' => 'random'),
      ),
    ),
  );
  $form['ls_invitation_options']['answ_title']['suffix']['random_group']['ls_suffix_max'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximal number'),
    '#default_value' => $answer_options['ls_suffix_max'],
    '#size' => '5',
    '#maxlength' => '10',
    '#suffix' => '</div>', // End of the "form-radios" style and the div container.
    '#states' => array(
      // Hide the settings.
      'visible' => array(
        ':input[name="ls_suffix_method"]' => array('value' => 'random'),
      ),
    ),
  );
  // Answer title example
  if ($ls_sid) {
    $form['ls_invitation_options']['answ_title']['suffix']['ls_suffix_example'] = array(
      '#type' => 'markup',
      // @TODO
      '#markup' => t('For example : %title', array('%title' => limesurvey_sync_api_response_create_title($ls_sid))),
    );
  }

  $form['ls_invitation_options']['answ_title']['ls_disabled_title'] = array(
    '#type' => 'checkbox',
    '#title' => t('The responses title can not be changed'),
    '#default_value' => (!empty($ls_sid) && $survey_properties && $survey_properties['anonymized']) ? 1 : $answer_options['ls_disabled_title'],
    '#return_value' => 1,
  );

  $token_table_exists = ($survey_properties && $survey_properties['tokens_table']);
  $token_tx = t('if you create the token table');
  $token_description = '';
  if ($default_options_page || !$ls_sid) {
    $create_token_link = $token_tx;
  }
  else {
    // Survey add/edit page and ls_sid is provided.
    if (!$token_table_exists) {
      $create_token_link = limesurvey_sync_link_to_ls($token_tx, $ls_sid, 'tokens');
    }
    else {
      $token_description = t('Thoses options are handled by the LimeSurvey token management system.');
    }
  }
  if (!$token_table_exists) {
    $token_description = t("Thoses options are availables only !create_token (also called : '<em>switch to closed-access mode</em>', or '<em>initialize token</em>', or '<em>create tokens table</em>'.", array('!create_token' => $create_token_link));
  }
  $form['ls_invitation_options']['token_management'] = array(
    '#type' => 'fieldset',
    '#title' => t('Options for the LimeSurvey token management feature'),
    '#description' => $token_description,
    '#collapsible' => 1,
    '#collapsed' => !$token_table_exists,
  );
  if (!$token_table_exists) {
    $form['ls_invitation_options']['token_management']['#description'] = "<strong>{$form['ls_invitation_options']['token_management']['#description']}</strong>";
  }

  $fieldnames = array('none' => t('Leave it empty'));
  $fieldnames['name'] = t('Username');
  $fieldnames['mail'] = t('E-Mail');
  $fieldnames['roles'] = t('Roles');
  foreach (field_info_instances('user', 'user') as $key => $value) {
    $fieldnames[$key] = $value['label'] . '  (' . $value['field_name'] . ')';
  }
  $disabled = (!($default_options_page || $token_table_exists));
  $form['ls_invitation_options']['token_management']['use_secure_token'] = array(
    '#type' => 'checkbox',
    '#default_value' => $answer_options['use_secure_token'],
    '#title' => t('Use secure token'),
    '#description' => t('create a token valid for a short period of time, just for answering the survey from your Drupal Site, for 2hours as an example.'),
    '#disabled' => $disabled,
  );

  $form['ls_invitation_options']['token_management']['secure_validuntil_period'] = array(
    '#type' => 'select',
    '#title' => t('Choose expiration period for completing the survey'),
    '#default_value' => $answer_options['secure_validuntil_period'],
    '#description' => t('Since printing the survey to its final submission'),
    '#options' => array(
      600 => t('10 min'),
      3600 => t('1 hour'),
      7200 => t('2 hours'),
      86400 => t('1 day'),
      'custom' => t('Custom'),
    ),
    '#states' => array(
      // Hide the settings.
      'visible' => array(
        ':input[name="use_secure_token"]' => array('checked' => TRUE),
      ),
    ),
  );
  $form['ls_invitation_options']['token_management']['secure_validuntil_period_custom'] = array(
    '#type' => 'textfield',
    '#default_value' => $answer_options['secure_validuntil_period_custom'],
    '#title' => 'Choose a duration in seconds for filling the survey',
    //'#description' => '',
    '#size' => 8,
    '#disabled' => $disabled,
    '#suffix' => t('In seconds.'),
    '#states' => array(
      // Hide the settings.
      'visible' => array(
        ':input[name="use_secure_token"]' => array('checked' => TRUE),
        ':input[name="secure_validuntil_period"]' => array('value' => 'custom'),
      ),
    ),
  );

  $form['ls_invitation_options']['token_management']['ls_import_dp2ls_datas'] = array(
    '#type' => 'checkbox',
    '#default_value' => $answer_options['ls_import_dp2ls_datas'],
    '#title' => t('Import Drupal users informations into the LimeSurvey site'),
    '#description' => t('Add informations (as user email, name or any user field) into the LimeSurvey database; they will be availables when exporting datas from the LimeSurvey admin interface.'),
    '#disabled' => $disabled,
  );
  $form['ls_invitation_options']['token_management']['import_dp2ls_group'] = array(
    '#type' => 'fieldset',
    '#title' => t('Drupal user informations to import into the LimeSurvey site'),
    '#description' => '<strong>' . t('Warning : People accessing to the LimeSurvey admin interface will be able to view some confidential datas, users email for instance.') . '</strong>',
    '#states' => array(
      // Hide the settings.
      'visible' => array(
        ':input[name="ls_import_dp2ls_datas"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['ls_invitation_options']['token_management']['import_dp2ls_group']['email'] = array(
    '#type' => 'select',
    '#default_value' => $answer_options['email'],
    '#title' => t('Email'),
    '#options' => $fieldnames,
    '#disabled' => $disabled,
  );

  $form['ls_invitation_options']['token_management']['import_dp2ls_group']['firstname'] = array(
    '#type' => 'select',
    '#default_value' => $answer_options['firstname'],
    '#title' => t('First name'),
    '#options' => $fieldnames,
    '#disabled' => $disabled,
  );
  $form['ls_invitation_options']['token_management']['import_dp2ls_group']['lastname'] = array(
    '#type' => 'select',
    '#default_value' => $answer_options['lastname'],
    '#title' => t('Last name'),
    '#options' => $fieldnames,
    '#disabled' => $disabled,
  );

  // Load file for the limesurvey_sync_link_to_ls() function.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
  $ls_text_attribute = t('create attribute fields');
  $ls_link_attribute = (!$default_options_page) ? limesurvey_sync_link_to_ls($ls_text_attribute, $ls_sid, 'managetokenattributes') : $ls_text_attribute;
  $form['ls_invitation_options']['token_management']['import_dp2ls_group']['usesleft'] = array(
    '#type' => 'textfield',
    '#default_value' => $answer_options['usesleft'],
    '#title' => 'Uses left',
    //'#description' => '',
    '#size' => 4,
    '#disabled' => $disabled,
    '#suffix' => t('For importing more datas, !ls_link on the LimeSurvey admin interface and <a href="@dp_url">fields on Drupal users</a>.', array('!ls_link' => $ls_link_attribute, '@dp_url' => url('admin/config/people/accounts/fields'))),
  );

  // Variable attribute fields.
  $attribute = array(
    '#type' => 'select',
    '#options' => $fieldnames,
    '#disabled' => $disabled,
  );

  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  if ($ls_sid) {
    $this_survey = limesurvey_sync_survey_properties($ls_sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $form['ls_invitation_options']['token_management']['import_dp2ls_group'][$attribute_field] = $attribute;
      $form['ls_invitation_options']['token_management']['import_dp2ls_group'][$attribute_field]['#title'] = $datas['description'];
      $form['ls_invitation_options']['token_management']['import_dp2ls_group'][$attribute_field]['#description'] = t('Correspond to the data %field.', array('%field' => $attribute_field));
      $form['ls_invitation_options']['token_management']['import_dp2ls_group'][$attribute_field]['#default_value'] = $answer_options[$attribute_field];
    }
  }

  // The 'reset to default' button has been removed from D6 to D7 into the
  // node_type_form() function, we add it here.
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save options'),
    '#weight' => 40,
  );

  $form['actions']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
    '#weight' => 50,
  );

  return $form;
}

/**
 * Validate the answers content type options.
 */
function limesurvey_sync_invitation_options_form__validate($form, &$form_state) {

  // Final datas will be stored into the $new_ls_data array.
  $new_ls_datas = array();
  // Retrieve the fresh ls_sid. This function must be called after the
  // limesurvey_sync_survey_sid_form_validate() function.
  // Once defined for a survey, ls_sid can not be changed, so toke the defined
  // first, then the provided.
  $ls_sid = (!empty($form_state['values']['ls_sid'])) ? $form_state['values']['ls_sid'] : ((!empty($form_state['values']['surveys']['ls_sid'])) ? $form_state['values']['surveys']['ls_sid'] : FALSE);

  // $form_state['entity_type'] is set by the entity_ui_get_form() function.
  $survey_entityname = limesurvey_sync_api_entityname('survey');
  $default_options_page = (empty($form_state['entity_type']) || $form_state['entity_type'] != $survey_entityname);

  $form_racine = (!isset($form['ls_invitation_options'])) ? $form : $form['ls_invitation_options'];

  $frames = array('iframe', 'colorbox', 'lightbox2', 'modalframe');
  $frames_set = in_array(trim($form_state['values']['ls_redirect']), $frames);

  if (!$default_options_page) {
    // Load survey datas.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = limesurvey_sync_survey_properties($ls_sid);
    $correct_back_url = limesurvey_sync_api_response_check_ls_back_urls($ls_sid);
    // The end url may be overwritten into the submit process.
    $back_url_fail = (empty($form_state['values']['ls_overwrite_endurl']) && !$correct_back_url);
  }
  else {
    $back_url_fail = FALSE;
  }

  // Validate all fields one by one.
  $keys = limesurvey_sync_invitation_options_default($ls_sid);
  foreach ($keys as $key => $default_value) {
    $value = (isset($form_state['values'][$key])) ? $form_state['values'][$key] : '';
    $tested_value = (!is_array($value)) ? trim($value) : $value;
    // Save it !
    $new_ls_datas[$key] = $tested_value;
    $condition = TRUE;
    $datas = $form;
    $single_radio = FALSE;

    // Find fieldset :
    switch ($key) {
      case 'iframe_dim':
        // Values can be 'custom' or 'auto' or 'invisible'.
        $datas = ($tested_value == 'auto' || $tested_value == 'invisible') ?  $form_racine['answ_redirect']['iframe_dim'][$tested_value] : $form_racine['answ_redirect']['iframe_dim']['iframe_dim_group']['cutom'];
        $single_radio = TRUE;
        break;
      case 'iframe_dim_width':
      case 'iframe_dim_height':
        $datas = $form_racine['answ_redirect']['iframe_dim']['iframe_dim_group'];
        break;
      case 'ls_redirect':
      case 'ls_overwrite_endurl':
      case 'ls_hide_lang':
        $datas = $form_racine['answ_redirect'];
        break;
      case 'ls_end':
        // Values can be 'own' or 'page'.
        $datas = ($tested_value == 'page') ? $form_racine['answ_redirect']['ls_end']['page_group']['page'] : $form_racine['answ_redirect']['ls_end']['own'];
        $single_radio = TRUE;
        break;
      case 'ls_end_url':
        $datas = $form_racine['answ_redirect']['ls_end']['page_group'];
        break;
      case 'use_secure_token':
      case 'secure_validuntil_period':
      case 'secure_validuntil_period_custom':
      case 'ls_import_dp2ls_datas':
        $datas = $form_racine['token_management'];
        break;
      case 'email':
      case 'firstname':
      case 'lastname':
      case 'usesleft':
        $datas = $form_racine['token_management']['import_dp2ls_group'];
        break;
      case 'ls_prefix':
      case 'ls_disabled_title':
        $datas = $form_racine['answ_title'];
        break;
      case 'ls_suffix_method':
        $datas = ($tested_value == 'random') ? $form_racine['answ_title']['suffix']['random_group']['random'] : $form_racine['answ_title']['suffix']['numeric'];
        $single_radio = TRUE;
        break;
      case 'ls_suffix_min':
      case 'ls_suffix_max':
        $datas = $form_racine['answ_title']['suffix']['random_group'];
        break;
    }
    if (drupal_substr($key, 0, drupal_strlen('attribute_')) == 'attribute_') {
      $datas = $form_racine['token_management']['import_dp2ls_group'];
    }

    $element = (!$single_radio) ? $datas[$key] : $datas;

    // Check if datas are valid.
    switch ($element['#type']) {
      case 'checkbox':
        $condition = in_array($tested_value, array('', 0, 1));
        break;
      case 'select':
      case 'radios':
        $condition = array_key_exists($tested_value, $element['#options']);
        break;
      case 'checkboxes':
        if (!is_array($tested_value)) {
          $condition = FALSE;
        }
        else {
          $condition = (array_diff_key($tested_value, $element['#options']) == array());
        }
        break;
      case 'radio':
        switch ($key) {
          case 'iframe_dim':
            $availables = array('custom', 'auto', 'invisible');
            break;
          case 'ls_end':
            $availables = array('own', 'page');
            break;
          case 'ls_suffix_method':
            $availables = array('random', 'numeric');
            break;
          default:
            $availables = array();
        }
        if (!in_array($tested_value, $availables)) {
          $condition = FALSE;
        }
        break;
    }

    // Specific validations :

    // Handle the frame dimension validation.
    if ($key == 'iframe_dim' && $tested_value == 'invisible' && $form_state['values']['ls_redirect'] != 'iframe') {
      // @TODO : The key paramater does not hightlight if error.
      form_set_error($key, t("The invisible iframe feature is only available for opening the form into an iframe."));
    }
    elseif ($key == 'iframe_dim_width' || $key == 'iframe_dim_height') {
      if (!($form_state['values']['iframe_dim'] != 'custom' && empty($tested_value))) {
        $pattern = '#(^[0-9]{1,4}\.?[0-9]{0,1})[ ]?(%|px)?$#';
        if (!preg_match($pattern, drupal_strtolower($tested_value), $match)) {
          $condition = FALSE;
        }
        else {
          // rewrite the provided datas:
          $num = (float) $match[1];
          $unit = (!empty($match[2])) ? $match[2] : 'px';
          $type_dim = ($key == 'iframe_dim_width') ?  'width' :  'height';
          // Define limit values :
          $dim = array(
            'width' => array(
              '%' => array(
                'min' => 0,
                'max' => 100,
              ),
              'px' => array(
                'min' => 0,
                'max' => 3000,
              ),
            ),
            'height' => array(
              '%' => array(
                'min' => 0,
                'max' => 100,
              ),
              'px' => array(
                'min' => 0,
                'max' => 6000,
              ),
            ),
          );
          if ($unit == '%' && ($form_state['values']['ls_redirect'] == 'modalframe' || $form_state['values']['ls_redirect'] == 'lightbox2') && $form_state['values']['iframe_dim'] == 'custom') {
            form_set_error($key, t("The frame module does not support percent parameter, tip a pixel value or '0' for automatically fit it."));
          }
          elseif ($num < $dim[$type_dim][$unit]['min'] || $num > $dim[$type_dim][$unit]['max']) {
            form_set_error($key, t('The frame %dim must be between @dim_min@dim_unit and @dim_max@dim_unit', array(
              '%dim' => $type_dim,
              '@dim_min' => $dim[$type_dim][$unit]['min'],
              '@dim_max' => $dim[$type_dim][$unit]['max'],
              '@dim_unit' => $unit,
            )));
          }
          // Rewrite the provided datas:
          // Store width and height as '88.8%' or '600px'.
          // Keep it as '0' for automatic fit.
          if ($tested_value != '0') {
            $form_state['values'][$key] = ((string) $num) . $unit;
          }
        }
      }
    }

    if ($key == 'ls_redirect' && $frames_set) {
      if ($tested_value == 'colorbox' && !module_exists('colorbox')) {
        form_set_error($key, t("The '%module' module needs to be installed", array('%module' => 'colorbox')));
      }
      elseif ($tested_value == 'lightbox2' && !module_exists('lightbox2')) {
        form_set_error($key, t("The '%module' module needs to be installed", array('%module' => 'lightbox2')));
      }
      elseif ($tested_value == 'modalframe' && !module_exists('modalframe')) {
        form_set_error($key, t("The '%module' module needs to be installed", array('%module' => 'Modal Frame API')));
      }
    }

    if ($key == 'ls_hide_lang' && $tested_value && !module_exists('locale')) {
      form_set_error($key, t("The '%module' module needs to be installed", array('%module' => 'Locale')));
    }

    // Survey properties specific validations.
    if ($ls_sid) {
      if ($key == 'ls_redirect') {
        if ($back_url_fail) {
          if ($frames_set) {
            form_set_error($key, t('In order to automatically close the frame, you should :') . format_string('!do_this', array('!do_this' => limesurvey_sync_invitation_options_help_iframe($ls_sid, TRUE))));
          }
          else {
            // Don't throw an error message but a warning message:
            drupal_set_message(t('In order to access to advanced features (automatically import answers, redirect the user on survey completion), you should :') . format_string('!do_this', array('!do_this' => limesurvey_sync_invitation_options_help_iframe($ls_sid, FALSE))), 'warning');
          }
        }
      }

      if ($key == 'ls_end' && $tested_value == 'own' && $back_url_fail) {
        form_set_error($key, t('In order to be redirected to the response view page, you need to :') . format_string('!do_this', array('!do_this' => limesurvey_sync_invitation_options_help_iframe($ls_sid, TRUE))));
      }
      elseif ($key == 'ls_import_dp2ls_datas' && $tested_value) {
        $token_table_exists = ($survey_properties && $survey_properties['tokens_table']);
        if (!$token_table_exists) {
          form_set_error($key, t('In order to save this site datas into Limesurvey, you need to create a token table on the LimeSurvey admin interface.'));
        }
        elseif (!$survey_properties['tokenanswerspersistence']) {
          form_set_error($key, t('Only LimeSurvey surveys with token-based response persistence enabled can export users informations'));
        }
      }
      elseif ($key == 'ls_keep_revisions' && $tested_value) {
        if (!$survey_properties['can_edit_answer']) {
          form_set_error($key, t('Only surveys that can edit answers may generate revisions'));
        }
      }

      if ($key == 'ls_recycling' && $tested_value && (!$survey_properties['tokenanswerspersistence'] || !$survey_properties['alloweditaftercompletion'])) {
        form_set_error($key, t('Only surveys with token-based response persistence enabled may generate virgin answers'));
      }

    }
    elseif ($default_options_page) {
      // This is the survey content type default page : don't allow to store
      // changes which can not be checked, because the lack of the ls_sid.

      // Don't allow options that it depends on the LS end urls.
      if ($key == 'ls_redirect' && in_array($tested_value, $frames)) {
        form_set_error($key, t('You can not edit the default redirection.'));
      }
      elseif ($key == 'ls_end' && $tested_value == 'own') {
        form_set_error($key, t('You can not choose to redirect the user to its response page by default.'));
      }
      elseif ($key == 'ls_auto_import' && $tested_value) {
        form_set_error($key, t('You can not choose the automatic import feature by default.'));
      }
      elseif ($key == 'ls_import_dp2ls_datas' && $tested_value) {
        // Because this option has security implications, it cas not be TRUE by
        // default.
        form_set_error($key, t("The Drupal datas can not be imported into LimeSurvey by default."));
      }
      elseif ($key == 'ls_prefix' && $tested_value) {
        // Prefix will be add by JS on survey creation thanks to the
        // limesurvey_sync_survey_sid_form() function.
        form_set_error($key, t("You can not choose a defaut response title prefix."));
      }
    }

    if ($key == 'ls_recycling' && $tested_value && !$form_state['values']['ls_validuntil']) {
      form_set_error('ls_validuntil', t('You can not ask for recycling answers with an unlimited expiration time'));
    }

    // Generate unspecific errors if needed.
    if (!$condition) {
      $option = $element['#title'];
      form_set_error($key, t("The '%setting' setting is not valid.", array('%setting' => $option)));
    }
    if ($key == 'usesleft' && (!is_numeric($tested_value) || $tested_value < 1)) {
      form_set_error($key, t("The 'Uses left' parameter should be an integer (at least 1)."));
    }
    elseif ($key == 'usesleft') {
      // Integer value.
      $new_ls_datas[$key] = (int) $new_ls_datas[$key];
    }
    elseif ($key == 'usesleft') {
      form_set_error($key, t("The 'Uses left' parameter should be an integer (at least 1)."));
    }

    if ($key == 'secure_validuntil_period_custom' && $form_state['values']['secure_validuntil_period'] == 'custom' && (!is_numeric($tested_value) || $tested_value < 10)) {
      form_set_error($key, t("The 'token period' parameter should be an integer (at least 10s)."));
    }
  }

  // Import the selected le_options into the $form_state variable.
  $form_state['values']['ls_options'] = $new_ls_datas;
}

/**
 * Submit the answers content type options.
 *
 * This function creates or resets the associated answers content type and
 * saves options values into a drupal variable.
 */

function limesurvey_sync_invitation_options_form_submit($form, &$form_state) {

  $new_ls_options = $form_state['values']['ls_options'];
  $ls_sid = (!empty($form_state['values']['ls_sid'])) ? $form_state['values']['ls_sid'] : ((!empty($form_state['values']['surveys']['ls_sid'])) ? $form_state['values']['surveys']['ls_sid'] : FALSE);
  // $form_state['entity_type'] is set by the entity_ui_get_form() function.
  $survey_entityname = limesurvey_sync_api_entityname('survey');
  $default_options_page = (empty($form_state['entity_type']) || $form_state['entity_type'] != $survey_entityname);

  if (!$default_options_page) {
    $variable_name = "limesurvey_sync_survey_{$ls_sid}_options";
    $old_ls_options = variable_get($variable_name, array());
    variable_set($variable_name, $new_ls_options);
    module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
    limesurvey_sync_custom_watchdog($variable_name, $old_ls_options, $new_ls_options, 'limesurvey_sync');

    if (!empty($new_ls_options['ls_overwrite_endurl']) && !limesurvey_sync_api_response_check_ls_back_urls($ls_sid)) {
      // Reset the end url according to the iframe url.
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
      $success = limesurvey_sync_save_endurl($ls_sid);
      if ($success) {
        drupal_set_message(t('LimeSurvey end url has been edited.'));
      }
    }
  }
  else {
    $variable_name = 'limesurvey_sync_survey_default_options';
    variable_set($variable_name, $new_ls_options);
    drupal_set_message(t('LimeSurvey surveys default options has been saved.'));
    module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
    limesurvey_sync_custom_watchdog($variable_name, $old_ls_options, $new_ls_options, 'limesurvey_sync');
  }
}

/**
 * Display an html help text for setting the end url.
 *
 * @param $ls_sid
 *   A numeric (optional). The survey ID. Default is NULL
 * @param $type
 *   A string (optional). The answer content type machine name. Default is
 *   FALSE corresponding to the text : 'your_answers_content_type_value'.
 * @param $iframes
 *   A Boolean (optional). If set to TRUE, display the end url for iframes
 *   feature (add '/back' to the end url). Default is FALSE.
 * @param $new_window
 *   A Boolean (optional). If set to TRUE, open links into a new window.
 *   Default is TRUE.
 *
 * @return
 *   An html string displaying helping text for configuring the end url.
 */
function limesurvey_sync_invitation_options_help_iframe($ls_sid = FALSE, $iframes = FALSE, $new_window = TRUE) {

  $li = $lii = array();
  $li[] = t('simply check the checkbox : Automatically overwrites the LimeSurvey end url');
  $load_url_tx = t('automatically load URL');
  $end_url_tx = t('End URL');
  if (empty($ls_sid)) {
    $link_settings_end_url = $end_url_tx;
    $link_settings_autoredirect = $load_url_tx;
    $url_backfromsurvey = url(limesurvey_sync_api_response_lsback_path() . '/ls-survey-XXXXXX', array('absolute' => TRUE));
    $lii[] = t("'!link' to '%end_url' (or an equivalent relative url),", array('!link' => $link_settings_end_url, '%end_url' => $url_backfromsurvey));
  }
  else {
    // Load file for the limesurvey_sync_link_to_ls() function.
    module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
    $link_settings_autoredirect = limesurvey_sync_link_to_ls($load_url_tx, $ls_sid, 'editsurveysettings', 'presentation', TRUE, $new_window);
    $ls_back_urls = limesurvey_sync_api_response_ls_back_urls($ls_sid);
    foreach ($ls_back_urls as $ls_lang => $ls_back_url) {
      $link_settings_end_url = limesurvey_sync_link_to_ls($end_url_tx, $ls_sid, 'editsurveylocalesettings', limesurvey_sync_localsettings_anchor($ls_sid, $ls_lang), TRUE, $new_window);
      $lii[] = t("for %lang language : '!link' to '%end_url' (or an equivalent relative url),", array(
        '!link' => $link_settings_end_url,
        '%end_url' => $ls_back_url,
        '%lang' => limesurvey_sync_api_display_field('survey', 'ls_lang', $ls_lang),
      ));
    }
  }

  if ($iframes) {
    $show_autoredirect = TRUE;
    if ($ls_sid) {
      if (!empty($survey_properties['autoredirect'])) {
        $show_autoredirect = FALSE;
      }
    }
    if ($show_autoredirect) {
      $lii[] = t("and set '!link when survey complete ?' to <em>Yes</em> (optional).", array('!link' => $link_settings_autoredirect));
    }
  }

  $li[] = array(
    'data' => t("or manually set on your LimeSurvey admin user interface :"),
    'children' => $lii,
  );
  return theme('item_list', array('items' => $li));
}
