<?php

/**
 * @file
 * functions for the survey admin entities
 */

/**
 * Build form for selecting the ls_sid value on the entity edit form.
 *
 * This function displays information from the LimeSurvey database, allowing
 * the user to pick a survey Results will be themed by the
 * theme_limesurvey_sync_survey_sid_form(() function.
 *
 * @param $form_state
 *   The editing form.
 * @param $ls_survey
 *   The survey entity object.
 *
 * @return
 *   An array. The drupal formated form.
 */
function limesurvey_sync_survey_sid_form($form, &$form_state, $ls_survey, $short = FALSE) {
  $ls_sid_default = '';
  // Find ls_sid, retreive the provided user value or the default survey
  // value if empty.
  if (isset($form_state['input']['surveys']['ls_sid']) && trim($form_state['input']['surveys']['ls_sid']) != '' && is_numeric(trim($form_state['input']['surveys']['ls_sid']))) {
    $ls_sid_default = $form_state['input']['surveys']['ls_sid'];
  }
  elseif (!empty($ls_survey->ls_sid) && is_numeric(trim($ls_survey->ls_sid))) {
    $ls_sid_default = $ls_survey->ls_sid;
  }

  // Load file for displaying boolean images.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');

  $form = array();

  // Add a css file for the compatibility table.
  $form['#attached']['css'] = array(
    drupal_get_path('module', 'limesurvey_sync') . '/misc/limesurvey_sync_survey.css',
  );

  $form['surveys'] = array(
    '#type' => 'fieldset',
    '#title' => t('Surveys from your LimeSurvey site'),
    '#description' => t('Choose a survey from your LimeSurvey site'),
    '#weight' => -1,
    '#element_validate' => array(
      'limesurvey_sync_survey_sid_form_validate',
    ),
    '#tree' => 1,
  );
  // The menu fiedset #weight is -2, the survey fieldset will be upper the
  // menu fieldset.
  $form['surveys']['admin'] = array(
    '#type' => 'markup',
    '#prefix' => '<p>',
    '#markup' => t('Go to the !link to change those options', array(
      '!link' => limesurvey_sync_link_to_ls(t('administration pages of your LimeSurvey site'), $ls_sid_default),
    )),
    '#suffix' => '</p>',
    '#weight' => -3,
  );
  if (!$short) {
    $form['surveys']['compatibility'] = array(
      '#type' => 'fieldset',
      '#title' => t('Help on LimeSurvey settings for specific features on this site'),
      '#weight' => -1,
      '#collapsible' => TRUE,
      '#collapsed' => (empty($ls_survey->is_new)),
      //'#tree' => 1,
    );

    $form['surveys']['compatibility']['help'] = array(
      '#type' => 'markup',
      '#markup' => limesurvey_sync_survey_compatibilityhelp($ls_sid_default),
    );
    $form['surveys']['compatibility']['table'] = array(
      '#type' => 'fieldset',
      '#title' => t('For custom use'),
      '#collapsible' => 1,
      '#collapsed' => 1,
    );
    $form['surveys']['compatibility']['table']['display_table'] = limesurvey_sync_survey_compatibility_table($ls_survey);
  }

  // Print all surveys availables from the LimeSurvey website.
  $k = 0;
  $js_tab = '';
  $radios = array();
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $all_surveys = limesurvey_sync_survey_properties('all');

  if ($all_surveys === FALSE) {
    // Avoid error message.
    $all_surveys = array();
  }
  $form_type = array(
    '#type' => 'markup',
  );
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  // for limesurvey_sync_invitation_fix_jquery() function :
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_invitation.form');

  foreach ($all_surveys as $ls_sid => $survey) {
    // Display only one survey if the sid is not defined as long as I disallow
    // to edit the survey sid.
    if (empty($ls_survey->is_new) && !empty($ls_survey->ls_sid) && $ls_sid != $ls_survey->ls_sid) {
      continue;
    }

    // Adapted version of the expand_radios() core function, in
    // include/form.inc file, line 1753.
    $disabled = (!$survey['active']);
    $ls_sid_parents = array(
      'surveys',
      'ls_sid',
    );
    $parents_for_id = array_merge($ls_sid_parents, array(
      $ls_sid,
    ));
    $id = drupal_clean_css_identifier('edit-' . implode('-', $parents_for_id));
    $labelfor = array(
      '#prefix' => '<label class="option" for="' . $id . '">',
      '#suffix' => '</label>',
    );

    $radios[$ls_sid] = array(
      '#type' => 'radio',
      '#title' => '', // The value will be dispayed by the "display" fieldset.
      '#return_value' => $ls_sid,
      '#default_value' => (!empty($ls_sid_default)) ? $ls_sid_default : FALSE,
      '#parents' => $ls_sid_parents,
      '#id' => $id,
      // Custom :
      // Only authorized survey can be checked.
      '#disabled' => 0, // ($disabled) ? 1 : 0,
      // Available surveys are on the top.
      '#weight' => ($disabled) ? 1 : ((!empty($ls_sid_default) && $ls_sid_default == $ls_sid) ? -1 : 0),
    );
    if (!empty($ls_survey->is_new)) {
      // Pre fill by JS the label for new survey.
      // $var.click(); does not help to launch Drupal.Behaviors.'machineName'.
      $label = str_replace("'", "\'", "{$survey['#lang'][$survey['default_language']]['title']} ({$ls_sid})");
      $prefix_title = str_replace("'", "\'", "{$survey['#lang'][$survey['default_language']]['title']} #");
      $ls_description = str_replace("'", "\'", str_replace("\t", '', preg_replace("/\r\n|\r|\n/", '<br/>', $survey['#lang'][$survey['default_language']]['description'])));
      $js_function = "\$var = document.getElementById('edit-label'); \$var.value = '$label';
        \$var2 = document.getElementById('edit-ls-prefix'); \$var2.value = '$prefix_title';
        \$var3 = document.getElementById('edit-description'); \$var3.value = '$ls_description';
        \$var.focus();"; // Drupal.Behaviors.'machineName'
      $radios[$ls_sid]['#attributes'] = array(
        'onclick' => limesurvey_sync_invitation_fix_jquery($js_function),
      );
    }

    // Add form elements.
    $form['surveys']['display'][$ls_sid]['sid'] = array_merge($form_type, array(
      '#markup' => limesurvey_sync_api_display_field('survey', 'ls_sid', $ls_sid),
    ), $labelfor);
    $form['surveys']['display'][$ls_sid]['active'] = array_merge($form_type, array(
      '#markup' => limesurvey_sync_display_boolean_img($survey['active']),
    ), $labelfor);
    $display_lang = $survey['default_language'];
    $form['surveys']['display'][$ls_sid]['title_survey'] = array_merge($form_type, array(
      '#markup' => '<strong>' . $survey['#lang'][$display_lang]['title'] . '</strong>',
    ), $labelfor);
    $form['surveys']['display'][$ls_sid]['description_survey'] = array_merge($form_type, array(
      '#markup' => $survey['#lang'][$display_lang]['description'],
    ), $labelfor);

    $nbr_lang[$ls_sid] = count($survey['#lang']);
    $ls_languages = array();
    foreach ($survey['#lang'] as $lg => $field) {
      $language = limesurvey_sync_api_display_field('survey', 'ls_lang', $lg, 'value', $ls_sid);
      if ($nbr_lang[$ls_sid] > 1 && $lg == $survey['default_language']) {
        $language = "<strong>$language</strong>";
      }
      $ls_languages[] = $language;
    }
    sort($ls_languages);
    $form['surveys']['display'][$ls_sid]['lang_survey'] = array_merge($form_type, array(
      '#markup' => implode(', ', $ls_languages),
    ), $labelfor);
    // entity rights will be check below.
    $affected_survey = limesurvey_sync_survey_get_survey_by_sid($ls_sid);
    if ($affected_survey) {
      // This survey already exists in the Drupal website.
      // Print it only if allowed.
      $affected = (entity_access('view', 'limesurvey_sync_survey', $affected_survey)) ? l($affected_survey->label, current(entity_uri('limesurvey_sync_survey', $affected_survey))) : t('Not available');
    }
    else {
      $affected = ($disabled) ? t('Not available') : t('Available');
    }
    $form['surveys']['display'][$ls_sid]['affected'] = array_merge($form_type, array(
      '#markup' => $affected,
    ), $labelfor);
  }

  $form['surveys']['ls_sid'] = $radios;

  $form['surveys']['#theme'] = 'limesurvey_sync_survey_sid_form';

  return $form;
}

/**
 * Validate the ls_sid and ls_lang values when editing a node survey.
 *
 * This function makes sure the survey ID reference and langage are valid.
 * However, display warning message instead of error message if
 * synchronization fails, for archive raisons.
 *
 * @param $limesurvey_sync_survey
 *   The survey entity as an object, from the entity edit form.
 * @param $form
 *   An array. The survey entity edit form.
 */
function limesurvey_sync_survey_sid_form_validate($form, &$form_state) {

  // Import the selected sid into the main form. It will be validated there
  $ls_sid = check_plain($form_state['values']['surveys']['ls_sid']);
  $form_state['values']['ls_sid'] = check_plain($form_state['values']['surveys']['ls_sid']);
}

/**
 * Display properties of all surveys as an html table.
 */
function theme_limesurvey_sync_survey_sid_form($variables) {

  $form = $variables['form'];
  $rows = $row = $header = array();
  foreach (element_children($form['ls_sid'], FALSE) as $ls_sid) {
    $row = array();
    $row[] = drupal_render($form['ls_sid'][$ls_sid]);
    $row[] = drupal_render($form['display'][$ls_sid]['sid']);
    $row[] = drupal_render($form['display'][$ls_sid]['title_survey']);
    $row[] = drupal_render($form['display'][$ls_sid]['affected']);
    $row[] = drupal_render($form['display'][$ls_sid]['lang_survey']);
    $row[] = drupal_render($form['display'][$ls_sid]['description_survey']);
    $row[] = drupal_render($form['display'][$ls_sid]['active']);
    $rows[] = $row;
  }
  $header[] = array(
    'data' => t('Survey ID'),
    'colspan' => 2, // Sid radio button.
  );
  $header[] = array(
    'data' => t('Title'),
  );
  $header[] = array(
    'data' => t('Affected to'),
  );
  $header[] = array(
    'data' => t('languages'),
  );
  $header[] = array(
    'data' => t('Description'),
  );
  $header[] = array(
    'data' => t('Activated'),
  );

  if (!count($rows)) {
    // Load file for the limesurvey_sync_link_to_ls() function.
    module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
    $row[] = array(
      'data' => t('No survey was found : !link_create or check the !link_settings',
          array(
        '!link_create' => limesurvey_sync_link_to_ls(t('create surveys'), NULL, 'newsurvey'),
        '!link_settings' => l(t('synchronization settings of your LimeSurvey site'), 'admin/structure/limesurvey/settings'),
      )),
      'colspan' => 7,
      'style' => 'text-align:center',
    );
    $rows[] = $row;
  }
  $output = theme('table', array(
    'header' => $header,
    'rows' => $rows,
  ));
  return $output . drupal_render_children($form);
}

/**
 * Display the node_survey view page.
 *
 * Display table including informations about this survey from the LimeSurvey
 * website and this site.
 *
 * @param $limesurvey_sync_survey
 *   The survey node object.
 *
 * @return
 *   An html table with all datas.
 */
function limesurvey_sync_survey_view_properties($limesurvey_sync_survey) {

  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $survey_properties = limesurvey_sync_survey_properties($limesurvey_sync_survey->ls_sid);

  // Load file for displaying boolean images.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
  $survey_valid = (!empty($survey_properties));
  $answ_type = $limesurvey_sync_survey->type;
  $row = $rows = $headers = array();
  // From node.css.
  $error_class_td = FALSE; // switch.
  $error_class = 'revision-current'; // background yellow.
  $error_class2 = 'node-unpublished'; // background pink.

  // Set some texts used several times.
  $no_more_survey = '<strong>' . t('The LimeSurvey survey %sid does not exist anymore on your !link !', array(
    '%sid' => limesurvey_sync_api_display_field('survey', 'ls_sid', $limesurvey_sync_survey->ls_sid),
    '!link' => limesurvey_sync_link_to_ls('LimeSurvey admin interface'),
  )) . '</strong>';
  $edit = t('edit');

  $introduction = '<p>' . t('All functionalities may not be availables from this drupal site, depending on specific survey settings from your LimeSurvey admin interface.') . '<br /><strong>' . t(
      'The table below shows which features are availables and how making then availables, setting each LimeSurvey parameter using the [edit] link into the right colomn.') . '</strong></p>';
  $headers[] = array(
    'data' => t('From this site'),
    'colspan' => 2,
  );
  $link = ($survey_valid) ? ' [' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid) . ']' : '';
  $headers[] = array(
    'data' => t('On the LimeSurvey admin interface') . $link,
    'colspan' => 4,
  );

  $row = array();

  $answers_creatable = ($survey_valid && $survey_properties['is_available']);
  $row[] = array(
    'data' => $answers_creatable ? t('Answers can be <strong>created</strong>') : t('Answers <strong>can not be created</strong>'),
    'rowspan' => ($survey_valid) ? 3 : 1,
  );
  if ($error_class_td && !$answers_creatable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  $row[] = array(
    'data' => limesurvey_sync_display_boolean_img($answers_creatable, FALSE),
    'rowspan' => ($survey_valid) ? 3 : 1,
  );
  if ($error_class_td && !$answers_creatable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  if (!$survey_valid) {
    $row[] = array(
      'data' => $no_more_survey,
      'rowspan' => 3,
      'colspan' => 4,
    );
  }
  else {
    $row[] = array(
      'data' => ($survey_properties['active']) ? t('The survey is activated') : '<strong>' . t('Because the survey is not activated') . '</strong>',
      'colspan' => 2,
    );
    $row[] = limesurvey_sync_display_boolean_img($survey_properties['active'], FALSE);
    $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'activate', 'general') . ']';
    $rows[] = $row;
    $row = array();
    $row[] = (empty($survey_properties['startdate']) || $survey_properties['startdate'] < REQUEST_TIME) ? t('Start date is valid') : '<strong>' . t('Because the start date is not valid') . '</strong>';
    $row[] = (!empty($survey_properties['startdate'])) ? format_date($survey_properties['startdate'], 'short') : '-';
    $row[] = limesurvey_sync_display_boolean_img((empty($survey_properties['startdate']) || $survey_properties['startdate'] < REQUEST_TIME), FALSE);
    $row[] = array(
      'data' => '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveysettings', 'publication') . ']',
      'rowspan' => 2,
    );
    $rows[] = $row;
    $row = array();
    $row[] = (empty($survey_properties['expires']) || $survey_properties['expires'] > REQUEST_TIME) ? t('Expiry date is valid') : '<strong>' . t('Because the expiry date is not valid') . '</strong>';
    $row[] = (!empty($survey_properties['expires'])) ? format_date($survey_properties['expires'], 'short') : '-';
    $row[] = limesurvey_sync_display_boolean_img((empty($survey_properties['expires']) || $survey_properties['expires'] > REQUEST_TIME), FALSE);
  }
  $rows[] = $row;
  $row = array();

  $answers_editable = ($survey_valid && $survey_properties['can_edit_answer']);
  $row[] = array(
    'data' => $answers_editable ? t('Answers can be <strong>edited</strong>') : t('Answers <strong>can not be edited</strong>'),
    'rowspan' => ($survey_valid) ? 3 : 1,
  );
  if ($error_class_td && !$answers_editable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  $row[] = array(
    'data' => limesurvey_sync_display_boolean_img($answers_editable, FALSE),
    'rowspan' => ($survey_valid) ? 3 : 1,
  );
  if ($error_class_td && !$answers_editable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  if ($survey_valid) {
    $arg = array(
      '%parameter' => t('Enable token-based response persistence'),
      '%value' => t('Yes'),
    );
    $row[] = array(
      'data' => ($survey_properties['tokenanswerspersistence']) ? t('%parameter is set to %value', $arg) : '<strong>' . t('Because %parameter is not set to %value', $arg) . '</strong>',
      'colspan' => 2,
    );
    $row[] = limesurvey_sync_display_boolean_img($survey_properties['tokenanswerspersistence'], FALSE);
    $row[] = array(
      'data' => '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveysettings', 'tokens') . ']',
      'rowspan' => 2,
    );
    $rows[] = $row;
    $row = array();
    $arg = array(
      '%parameter' => t('Allow multiple responses or update responses with one token'),
      '%value' => t('Yes'),
    );
    $row[] = array(
      'data' => ($survey_properties['alloweditaftercompletion']) ? t('%parameter is set to %value', $arg) : '<strong>' . t('Because %parameter is not set to %value', $arg) . '</strong>',
      'colspan' => ($survey_valid) ? 2 : 1,
    );
    $row[] = limesurvey_sync_display_boolean_img($survey_properties['alloweditaftercompletion'], FALSE);
    $rows[] = $row;
    $row = array();
    $row[] = array(
      'data' => ($survey_properties['tokens_table']) ? t('The token table has been created') : '<strong>' . t('Because the token table has not been created') . '</strong>',
      'colspan' => 2,
    );
    $row[] = limesurvey_sync_display_boolean_img($survey_properties['tokens_table'], FALSE);
    $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'tokens') . ']';
  }
  $rows[] = $row;
  $row = array();

  $correct_back_url = limesurvey_sync_api_response_check_ls_back_urls($limesurvey_sync_survey->ls_sid);
  $answers_importable = ($survey_valid && $survey_properties['date_answer'] && !$survey_properties['anonymized'] && $correct_back_url);
  // Count how many language
  $n_ls_lang = count($survey_properties['#lang']);
  $rowspan_end_url = ($correct_back_url) ? 1 : $n_ls_lang;
  $row[] = array(
    'data' => $answers_importable ? t('Answers can be automatically <strong>imported</strong>') : t('Answers <strong>can not be automatically imported</strong>'),
    'rowspan' => ($survey_valid) ? 2 + $rowspan_end_url : 1,
  );
  if ($error_class_td && !$answers_importable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  $row[] = array(
    'data' => limesurvey_sync_display_boolean_img($answers_importable, FALSE),
    'rowspan' => ($survey_valid) ? 2 + $rowspan_end_url : 1,
  );
  if ($error_class_td && !$answers_importable) {
    $row[max(array_keys($row))]['class'] = $error_class2;
  }
  if ($survey_valid) {
    $arg = array(
      '%parameter' => t('Anonymized responses'),
      '%value' => t('No'),
    );
    $row[] = array(
      'data' => (!$survey_properties['anonymized']) ? t('%parameter is set to %value', $arg) : '<strong>' . t('Because %parameter is not set to %value', $arg) . '</strong>',
      'colspan' => 2,
    );
    $row[] = limesurvey_sync_display_boolean_img(!$survey_properties['anonymized'], FALSE);
    $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveysettings', 'tokens') . ']';
    $rows[] = $row;

    $row = array();
    $arg = array(
      '%parameter' => t('Date stamp'),
      '%value' => t('Yes'),
    );
    $row[] = array(
      'data' => ($survey_properties['date_answer']) ? t('%parameter is set to %value', $arg) : '<strong>' . t('Because %parameter is not set to %value', $arg) . '</strong>',
      'colspan' => 2,
    );
    $row[] = limesurvey_sync_display_boolean_img($survey_properties['date_answer'], FALSE);
    $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveysettings', 'notification') . ']';

    $rows[] = $row;
    $row = array();
    // Count how many language
    $n_ls_lang = count($survey_properties['#lang']);
    $arg = array(
      '%parameter' => format_plural($n_ls_lang, 'End url', 'End urls'),
    );
    if ($correct_back_url) {

      $row[] = array(
        'data' => format_plural($n_ls_lang, '%parameter is correctly set', '%parameter are correctly set', $arg),
        'colspan' => 2,
      );
      $row[] = limesurvey_sync_display_boolean_img($correct_back_url, FALSE);
      $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveylocalesettings') . ']';
      $rows[] = $row;
      $row = array();
    }
    else {
      // Display each back url.
      // Retrieve human readable ls lang.
      $ls_langs = limesurvey_sync_api_get_ls_langs($limesurvey_sync_survey->ls_sid);
      $end_urls = limesurvey_sync_api_response_ls_back_urls($limesurvey_sync_survey->ls_sid);
      foreach ($end_urls as $ls_lang => $back_url) {
        $arg = array(
          '%lang' => $ls_langs[$ls_lang],
          '%url' => $back_url,
        );
        $row[] = t('End url');
        $ok_end_url = ($back_url == $survey_properties['#lang'][$ls_lang]['back_url']);
        if ($ok_end_url) {
          $row[] = t('%lang end url is correctly set', $arg);
        }
        else {
          $row[] = '<strong>' . t('%lang end url is not correctly set (should be %url)', $arg) . '</strong>';
        }
        $row[] = limesurvey_sync_display_boolean_img($ok_end_url, FALSE);
        $row[] = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveylocalesettings', limesurvey_sync_localsettings_anchor($limesurvey_sync_survey->ls_sid, $ls_lang)) . ']';
        $rows[] = $row;
        $row = array();
      }

    }
  }

  $output = theme('table', array(
    'header' => $headers,
    'rows' => $rows,
  ));

  $render = array(
    '#theme' => 'table',
    '#header' => $headers,
    '#rows' => $rows,
  );
  return $render;
  //return $introduction . $output;
}


/**
 * Display an html help text for compatibility with the limesurvey_sync_survey module.
 *
 * Provide correct values the survey language depending on the survey ID
 * reference provided and the authorized value available.
 *
 * @param $sid
 *   A numeric (optional). The survey ID reference. Default is NULL
 *
 * @return
 *   An html string displaying helping text for configuring the LimeSurvey
 *   website for limesurvey_sync_survey module compatibility.
 */
function limesurvey_sync_survey_compatibilityhelp($sid = FALSE) {
  // Load file for the limesurvey_sync_link_to_ls() function.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');

  if ($sid) {
    $ls_link = limesurvey_sync_link_to_ls(t('LimeSurvey admin interface'), $sid, 'editsurveysettings', 'presentation');
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = limesurvey_sync_survey_properties($sid);
  }
  else {
    $ls_link = limesurvey_sync_link_to_ls(t('LimeSurvey admin interface'), NULL, 'listsurveys');
  }
  $type = 'ls-survey-XXXXXX';
  $url_backfromsurvey = url(limesurvey_sync_api_response_lsback_path(), array('absolute' => TRUE)) . $type;

  $output = '<p>' . t('Note that this module does not override the LimeSurvey site settings. In order to access to specific features, you need to set appropiate survey settings <em>on the !ls_link</em>', array(
    '!ls_link' => $ls_link,
  )) . '</p>';
  $output .= '<p>' . t('In a nutshell :') . '</p>';
  //$list = array();
  $list = array(
    t('For a basic use : you only need to activate the survey.') . '</b>',
    array(
      'data' => t('For the most advanced use, you need to :'),
      'children' => array(
        t('enable token-based response persistence,'),
        t("set 'allow editing answers after completion' to <em>yes</em>,"),
        t("create the invitation token table,"),
        t("and set on <em>each language</em> the 'end url' to %end_url.", array('%end_url' => $url_backfromsurvey)),
      ),
    ),
    t('For a custom use : check below.'),
  );

  // Hard to provide this as render array : it comes throw a markup form
  // element and I don't find a way to do it, without writing a custum theme
  // only for this. Anyway, it it just information no customisation needed.
  $render = array(
    'introduction' => array(
      '#markup' => t('Note that this module does not override the LimeSurvey site settings. In order to access to specific features, you need to set appropiate survey settings <em>on the !ls_link</em>', array('!ls_link' => $ls_link)),
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    ),
    'In a nutshell' => array(
      '#markup' => t('In a nutshell :'),
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    ),
    'help_list' => array(
      '#theme' => 'item_list',
      //'#type' => 'li',
      '#items' => $list,
    ),
  );
  return $output . theme('item_list', array('items' => $list));
}

/**
 * Display a helper table for ls settings.
 *
 * @param $limesurvey_sync_survey
 *   The survey node object. Default is NULL (not provided).
 *
 * @return
 *   An html table with all datas.
 */
function limesurvey_sync_survey_compatibility_table($limesurvey_sync_survey = NULL) {
  // Load file for displaying boolean images.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
  $cells = $cell = $header = array();
  $center = array(
    'align' => 'center',
  );
  $empty_cell = '';
  $type = 'your_answers_content_type_value';
  if (!empty($limesurvey_sync_survey)) {
    if ($answ_type = $limesurvey_sync_survey->type) {
      // $info = entity_get_info('limesurvey_sync_survey');
      // $type = 'limesurvey_sync_answer';
    }
  }
  $type = 'ls-survey-XXXXXX';
  $url_backfromsurvey = url(limesurvey_sync_api_response_lsback_path(), array('absolute' => TRUE)) . $type;

  // Add css file for displaying the compatibility table borders.
  // drupal_add_css(drupal_get_path('module', 'limesurvey_sync') . 'survey/limesurvey_sync_survey.css', array('group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE));
  // Edit : The limesurvey_sync_survey.css is load into the limesurvey_sync_survey_sid() function.

  $sub_header = array(
    'header' => TRUE,
  );
  $no_striping = array(
    'no_striping' => TRUE,
  );
  $first_col = array(
    'class' => 'ls_first_col',
  );
  $header = array();
  $header[] = array(
    'data' => t('Settings on the LimeSurvey admin interface'),
    'colspan' => 2,
    'class' => 'ls_bd_top_left',
  );

  $header[] = array_merge(array(
    'data' => t('Activate survey'),
  ), array(
    'class' => 'ls_bd_top',
  ));
  $header[] = array_merge(array(
    'data' => t('Available'),
  ), array(
    'class' => 'ls_bd_top_right',
  ));
  $header[] = array_merge(array(
    'data' => t('Allow editing answers after completion'),
  ));
  $header[] = array_merge(array(
    'data' => t('Enable token-based response persistence'),
  ));
  $header[] = array_merge(array(
    'data' => t('Use token invitations'),
  ));
  $header[] = array_merge(array(
    'data' => t('End URL'),
  ));
  $header[] = array_merge(array(
    'data' => t('Date stamp'),
  ));
  $header[] = array_merge(array(
    'data' => t('Anonymized responses'),
  ));

  $cell[] = array_merge(array(
    'data' => '',
  ), array(
    'header' => TRUE,
    'class' => 'ls_bd_left',
  ));
  $cell[] = array_merge(array(
    'data' => t('Set to :'),
    'align' => 'right',
  ), $first_col);
  // array('data' => '', 'class' => 'ls_bd_left'); // Features colomns.
  // $cell[] = array_merge(array('data' => t('Set to :')), $center);
  $cell[] = array_merge(array(
    'data' => t('Currently active'),
  ), $center);
  $cell[] = array_merge(array(
    'data' => t('Current time is between the survey start date and expiry date (default setting)'),
  ), $center, array(
    'class' => 'ls_bd_right',
  ));
  $cell[] = array_merge(array(
    'data' => t('Yes'),
  ), $center);
  $cell[] = array_merge(array(
    'data' => t('Yes'),
  ), $center);
  $cell[] = array_merge(array(
    'data' => t("Yes (also called : '<em>switch to closed-access mode</em>', or '<em>initialize token</em>', or '<em>create tokens table</em>'"),
  ), $center);
  $end_url_text = $url_backfromsurvey;
  // Don't display recommanded setting for greater readibility.
  // $end_url_text .= '<br />' . t("(and as a recommanded setting, set '<em>Automatically load URL when survey complete</em>' to '<em>Yes</em>')");
  $cell[] = array_merge(array(
    'data' => $end_url_text,
  ), $center);
  $cell[] = array_merge(array(
    'data' => t('Yes'),
  ), $center);
  $cell[] = array_merge(array(
    'data' => t('No'),
  ), $center);
  // $cells[] = $cell;
  $cells[] = array_merge(array(
    'data' => $cell,
    'class' => 'ls_first_row',
  ), $no_striping);

  $cell = array();
  $cell[] = array_merge(array(
    'data' => '<strong>' . t('Basic features') . '</strong>',
    'rowspan' => 2,
  ), $sub_header, $center, array(
    'class' => 'ls_bd_bottom_left',
  ));
  $cell[] = array_merge(array(
    'data' => t('Create survey'),
  ), $first_col);
  $cell[] = limesurvey_sync_display_boolean_img(TRUE);
  $cell[] = array(
    'data' => $empty_cell,
    'class' => 'ls_bd_right',
  );
  $cell = array_merge($cell, array_fill(4, 6, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Create answers'),
    'class' => 'ls_bd_intersect',
  ), $no_striping);
  $cell[] = array(
    'data' => $empty_cell,
    'class' => 'ls_bd_bottom',
  );
  $cell[] = array(
    'data' => limesurvey_sync_display_boolean_img(TRUE),
    'class' => 'ls_bd_bottom_right',
  );
  $cell = array_merge($cell, array_fill(3, 6, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => '<strong>' . t('Advanced features') . '</strong>',
    'rowspan' => 7,
  ), $sub_header);
  $cell[] = array_merge(array(
    'data' => t('Edit answers'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(2, 2, $empty_cell));
  $cell = array_merge($cell, array_fill(4, 3, limesurvey_sync_display_boolean_img(TRUE)));
  $cell = array_merge($cell, array_fill(6, 3, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Import drupal users informations into the LimeSurvey results'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 3, $empty_cell));
  $cell = array_merge($cell, array_fill(4, 2, limesurvey_sync_display_boolean_img(TRUE)));
  $cell = array_merge($cell, array_fill(6, 3, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Display answer form into an iframe'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 5, $empty_cell));
  $cell[] = limesurvey_sync_display_boolean_img(TRUE);
  $cell = array_merge($cell, array_fill(7, 2, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Redirection to a drupal specific page after completion'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 5, $empty_cell));
  $cell[] = limesurvey_sync_display_boolean_img(TRUE) . t("Or directly set it as the end url if you don't use neither the auto import answers or the iframe features.");
  $cell = array_merge($cell, array_fill(7, 2, $empty_cell));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Import all answers manually'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 6, $empty_cell));
  $cell[] = array_merge(array(
    'data' => t('If set to %yes_no : after survey expiry date only', array(
      '%yes_no' => t('No'),
    )),
  ), $center);
  $cell[] = array_merge(array(
    'data' => t('If set to %yes_no : after survey expiry date only', array(
      '%yes_no' => t('Yes'),
    )),
  ), $center);
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Automatic import answers') . '<br />' . t('and User view own page after completion'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 5, $empty_cell));
  $cell = array_merge($cell, array_fill(6, 3, limesurvey_sync_display_boolean_img(TRUE)));
  $cells[] = $cell;

  $cell = array();
  $cell[] = array_merge(array(
    'data' => t('Generate numeric answer titles'),
  ), $first_col);
  $cell = array_merge($cell, array_fill(1, 7, $empty_cell));
  $cell[] = limesurvey_sync_display_boolean_img(TRUE);
  $cells[] = $cell;

  $render = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $cells,
  );
  return $render;
}

/**
 * Display on error or a simple message depending on a condition.
 *
 * This allow a node to be still editable (on revision, published status for
 * example) displaying warning message about desynchronization whereas error
 * messages would break node general edition.
 *
 * @param $error
 *   A Boolean. If TRUE, an error message is displayed. If FALSE, a warning
 *   message is displayed.
 * @param $field
 *   A string. The corresponding form field.
 * @param $message
 *   A string. The message to be displayed.
 * @param $type
 *  A string (optional). The type of the message. One of the following values
 *  are possible: 'status', 'warning', 'error'. Default to 'warning'.
 */
function limesurvey_sync_survey_error_or_message($error, $field, $message, $type = 'warning') {

  if (!empty($message)) {
    if ($error) {
      form_set_error($field, filter_xss($message));
    }
    else {
      drupal_set_message(filter_xss($message), $type);
    }
  }
}

/**
 * Display the node_survey view page.
 *
 * Display table including informations about this survey from the LimeSurvey
 * website and this site.
 *
 * @param $limesurvey_sync_survey
 *   The survey node object.
 *
 * @return
 *   An html table with all datas.
 */
function limesurvey_sync_survey_view_properties_languages($limesurvey_sync_survey, $full_viewmode = TRUE) {
  // Load file for displaying boolean images.
  module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
  module_load_include('inc', 'limesurvey_sync', 'ls_answ/limesurvey_sync_answer');
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $survey_properties = limesurvey_sync_survey_properties($limesurvey_sync_survey->ls_sid);
  $survey_valid = (!empty($survey_properties));
  if (!$survey_valid) {
    return '<strong>' . t('The survey %sid does not exist anymore on your !link !', array(
      '%sid' => limesurvey_sync_api_display_field('survey', 'ls_sid', $limesurvey_sync_survey->ls_sid),
      '!link' => limesurvey_sync_link_to_ls('LimeSurvey admin interface'),
    )) . '</strong>';
  }
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api');
  $previous_db = limesurvey_sync_ls_api_active_db_name();
  limesurvey_sync_ls_api_db_close();

  $answ_type = $limesurvey_sync_survey->type;
  $rows = $headers = array();
  $function_i18n_string = 'i18n_string_translate';
  $function_i18n_text = 'i18n_string_text';
  // @TODO : match better format.
  $in18_text_format = 'filtered_html';
  $in18_options = array(
    'format' => $in18_text_format,
  );
  $edit = t('edit');
  $windows_tag = array(
    'attributes' => array(
      'title' => t('Open into a new window'),
      'onclick' => 'window.open(this.href); return false;',
    ),
  );
  $site_default_language = language_default('language');

  $lang_dp2ls = limesurvey_sync_survey_mapping_dp2ls_lang($limesurvey_sync_survey->ls_sid);
  $col = $line = 0;
  if ($full_viewmode) {
    $headers[0] = array(
      'data' => 'Drupal enabled languages :',
      'colspan' => 3,
    );
  }
  $rows = array();
  $rows[0][0] = array(
    'data' => '<strong>' . t('LimeSurvey survey languages do not match each Drupal languages') . '</strong>',
  );
  // Resume colomn, set below.
  $rows[0][1] = array(
    'data' => limesurvey_sync_display_boolean_img(FALSE, FALSE),
    'colspan' => ($full_viewmode) ? 2 : 1,
  );
  $rows[1][0] = array(
    'data' => '<strong>' . t('LimeSurvey end URLs, redirecting to this site after completion, may not be correctly set') . '</strong>',
    'rowspan' => ($full_viewmode) ? 2 : 1,
  );
  // Resume colomn, set below.
  $rows[1][1] = array(
    'data' => limesurvey_sync_display_boolean_img(FALSE, FALSE),
    'rowspan' => ($full_viewmode) ? 2 : 1,
  );
  if ($full_viewmode) {
    $rows[1][2] = t('Recommanded for going back to this site');
    $rows[2][0] = t('currently set on your LimeSurvey site');
  }
  $rows[3][0] = array(
    'data' => '<strong>' . t('Title may not be correctly set') . '</strong>',
    'rowspan' => ($full_viewmode) ? 2 : 1,
  );
  $rows[3][1] = array(
    'data' => limesurvey_sync_display_boolean_img(NULL, FALSE),
    'rowspan' => ($full_viewmode) ? 2 : 1,
  );
  if ($full_viewmode) {
    $rows[3][2] = t('Drupal');
    $rows[4][0] = t('LimeSurvey');
  }
  $rows[5][0] = array(
    'data' => '<strong>' . t('Explanation or submission guidelines may not be correctly set') . '</strong>',
  );
  $rows[5][1] = array(
    'data' => limesurvey_sync_display_boolean_img(NULL, FALSE),
  );
  if ($full_viewmode) {
    $rows[5][2] = t('Drupal');
  }
  $rows[6][0] = array(
    'data' => '<strong>' . t('Description may not be correctly set') . '</strong>',
    'rowspan' => 2,
  );
  $rows[6][1] = array(
    'data' => limesurvey_sync_display_boolean_img(NULL, FALSE),
    'rowspan' => 2,
  );
  if ($full_viewmode) {
    $rows[6][2] = t('Drupal');
    $rows[7][0] = t('LimeSurvey');
  }
  $col1 = 1;
  $dp_guideline_edit_link = l($edit, 'admin/structure/types/manage/' . str_replace('_', '-', $answ_type->type), array_merge($windows_tag, array(
    'fragment' => 'edit-submission',
  )));

  $last_loop = FALSE;
  $last_colomn = count($lang_dp2ls) * 2 + 1;

  foreach ($lang_dp2ls as $dp_lang => $ls_lang) {
    $col1 = $col1 + 2;
    $col2 = $col1 + 1;
    $last_loop = $col1 >= $last_colomn;
    $survey_help = $answ_type->help;
    $survey_description = $answ_type->description;
    $all_tests[0][] = (!empty($ls_lang));
    $all_tests[1][] = (!isset($survey_properties['#lang'][$ls_lang]) || $survey_properties['#lang'][$ls_lang]['back_to_drupal']);
    $all_tests[3][] = (!isset($survey_properties['#lang'][$ls_lang]) || !empty($survey_properties['#lang'][$ls_lang]['title']));
    $all_tests[5][] = (!empty($survey_help));
    $all_tests[6][] = (!empty($survey_description));
    if ($last_loop) {
      if (count(array_unique($all_tests[0])) === 1 && $all_tests[0][0] === TRUE) {
        $rows[0][0]['data'] = t('LimeSurvey survey languages match each Drupal languages');
        $rows[0][1]['data'] = limesurvey_sync_display_boolean_img(TRUE, FALSE);
      }
      if (count(array_unique($all_tests[1])) === 1 && $all_tests[1][0] === TRUE) {
        $rows[1][0]['data'] = t('LimeSurvey end URLs are redirecting to this site after completion');
        $rows[1][1]['data'] = limesurvey_sync_display_boolean_img(TRUE, FALSE);
      }
      if (count(array_unique($all_tests[3])) === 1 && $all_tests[3][0] === TRUE) {
        $rows[3][0]['data'] = t('Title seems correctly set');
        $rows[3][1]['data'] = limesurvey_sync_display_boolean_img(((module_exists('i18n') && module_exists('i18n_string')) ? TRUE : NULL), FALSE);
      }
      if (count(array_unique($all_tests[5])) === 1 && $all_tests[5][0] === TRUE) {
        $rows[5][0]['data'] = t('Explanation or submission guidelines seems correctly set');
        $rows[5][1]['data'] = limesurvey_sync_display_boolean_img(((module_exists('i18n') && module_exists('i18n_string')) ? TRUE : NULL), FALSE);
      }
      if (count(array_unique($all_tests[6])) === 1 && $all_tests[6][0] === TRUE) {
        $rows[6][0]['data'] = t('Description seems correctly set');
        $rows[6][1]['data'] = limesurvey_sync_display_boolean_img(((module_exists('i18n') && module_exists('i18n_string')) ? TRUE : NULL), FALSE);
      }
    }

    if ($full_viewmode) {
      module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
      $headers[$col1] = array(
        'data' => t('@lang (%lang_code)', array(
          '@lang' => limesurvey_sync_api_language_name($dp_lang),
          '%lang_code' => $dp_lang,
        )),
        'colspan' => 2,
      );
      $ls_textedit_link = '[' . limesurvey_sync_link_to_ls($edit, $limesurvey_sync_survey->ls_sid, 'editsurveylocalesettings', limesurvey_sync_localsettings_anchor($limesurvey_sync_survey->ls_sid, $ls_lang)) . ']';
      $rows[0][$col1] = (!empty($ls_lang)) ? t('@lang (%lang_code)', array(
        '@lang' => limesurvey_sync_answer_display_sync_fields('ls_lang', $ls_lang),
        '%lang_code' => $ls_lang,
      )) : t('No matching LimeSurvey Language for the %lg Drupal language, !link',
          array(
        '%lg' => limesurvey_sync_survey_api_name($dp_lang),
        '!link' => limesurvey_sync_link_to_ls(t('create it !'), $limesurvey_sync_survey->ls_sid, 'editsurveysettings', 'general'),
      ));
      $rows[0][$col2] = limesurvey_sync_display_boolean_img((!empty($ls_lang)), FALSE);

      $rows[1][$col1] = (isset($survey_properties['#lang'][$ls_lang])) ? $survey_properties['#lang'][$ls_lang]['correct_back_url'] : '';
      $rows[1][$col2] = (isset($survey_properties['#lang'][$ls_lang])) ? limesurvey_sync_display_boolean_img($survey_properties['#lang'][$ls_lang]['back_to_drupal'], FALSE) : '';

      $rows[2][$col1] = (isset($survey_properties['#lang'][$ls_lang])) ? $survey_properties['#lang'][$ls_lang]['back_url'] : '';
      $rows[2][$col2] = (isset($survey_properties['#lang'][$ls_lang])) ? $ls_textedit_link : '';
      $survey_title = $limesurvey_sync_survey->label;
      if ($site_default_language == $dp_lang) {
        $dp_text_edit = $dp_guideline_edit_link;
      }
      else {
        if (module_exists('i18n') && !module_exists('i18n_string')) {
          $dp_text_edit = t('enable the <a href="@url">String translation</a>',
              array(
            '@url' => url('admin/modules', array(
              'fragment' => (!(module_exists('module_filter') ? 'i18n_string' : '#multilingual__internationalization')),
            )),
          ));
        }
        elseif (module_exists('i18n') && module_exists('i18n_string')) {
          if (function_exists($function_i18n_string)) {
            $translated = $function_i18n_string('node:type:' . $survey_properties['type'] . ':name', $survey_title, array(
              'langcode' => $dp_lang,
            ));
            $survey_title = (empty($translated)) ? t('Need translation') : $translated;
          }
          $id_tranlation = '';
          // @TODO : find $id_tranlation
          $dp_text_edit = l(t('Translate this string'), 'admin/config/regional/translate/edit/' . $id_tranlation);
        }
        else {
          $dp_text_edit = t('In order to edit it, install and enable the <a href="@url">Internationalization</a> module', array(
            '@url' => 'https://drupal.org/project/i18n',
          ));
        }
      }
      $rows[3][$col1] = $survey_title;
      $rows[3][$col2] = '[' . $dp_text_edit . ']';

      $rows[4][$col1] = (isset($survey_properties['#lang'][$ls_lang])) ? $survey_properties['#lang'][$ls_lang]['title'] : '';
      $rows[4][$col2] = (isset($survey_properties['#lang'][$ls_lang])) ? $ls_textedit_link : '';

      // Guidelines when submitting a new answers.
      $in18_options['langcode'] = $dp_lang;
      if (function_exists($function_i18n_text)) {
        $translated = $function_i18n_text('node:type:' . $survey_properties['type'] . ':help', $survey_help, $in18_options);
        $survey_description = (empty($translated)) ? t('Need translation') : $translated;
      }
      $rows[5][$col1] = $survey_help;
      $rows[5][$col2] = '[' . $dp_text_edit . ']';
      if (function_exists($function_i18n_text)) {
        $translated = $function_i18n_text('node:type:' . $survey_properties['type'] . ':description', $survey_description, $in18_options);
        $survey_description = (empty($translated)) ? t('Need translation') : $translated;
      }
      $rows[6][$col1] = $survey_description;
      $rows[6][$col2] = '[' . $dp_text_edit . ']';
      $rows[7][$col1] = (isset($survey_properties['#lang'][$ls_lang])) ? $survey_properties['#lang'][$ls_lang]['description'] : '';
      $rows[7][$col2] = (isset($survey_properties['#lang'][$ls_lang])) ? $ls_textedit_link : '';
    }
  }
  // @TODO : 'node:type:' . $survey_properties['type'] . ':label' to.
  $output = theme('table', array(
    'header' => $headers,
    'rows' => $rows,
  ));
  limesurvey_sync_ls_api_db_close($previous_db);
  return $output;
}

function limesurvey_sync_survey_mapping_dp2ls_lang() {

  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api');
  return limesurvey_sync_ls_api_fixlang_ls2dp('', 'enabled', TRUE);
}
